#!/usr/bin/python

import py2geom
import toyframework
import random,gtk
from py2geom_glue import *

class PathToy(toyframework.Toy):
    def __init__(self):
        toyframework.Toy.__init__(self)
        self.handles.append(toyframework.PointHandle(200, 200))
        self.path_b_name="star.svgd"
        self.pv = py2geom.read_svgd(self.path_b_name);
        centr = py2geom.Point()
        for p in self.pv:
            c,area = py2geom.centroid(p.toPwSb())
            centr += c
        self.pv = self.pv*py2geom.Matrix(py2geom.Translate(-centr))
    def draw(self, cr, pos, save):
        cr.set_source_rgba (0., 0., 0., 1)
        cr.set_line_width (1)
        
        
        B = (self.pv[0]*py2geom.Matrix(py2geom.Translate(*self.handles[0].pos))).toPwSb();
        n = py2geom.rot90(py2geom.unit_vector(py2geom.derivative(B), 0.01, 3));
        al = py2geom.arcLengthSb(B, 0.1);
        offset = 10.
        
        offset_curve = B+py2geom.sin(al*0.1, 0.01, 2)*n*10.
        offset_path = py2geom.path_from_piecewise(offset_curve, 0.1, True)
        
        py2geom.cairo_path(cr, offset_path)
        cr.stroke()
        
        self.notify = ''
        toyframework.Toy.draw(self, cr, pos, save)
        
t = PathToy()
import sys

toyframework.init(sys.argv, t, 500, 500)



from libcpp.vector cimport vector
from libcpp.pair cimport pair

ctypedef double Coord
ctypedef int IntCoord

cdef extern from "2geom/coord.h" namespace "Geom":
    cdef Coord EPSILON
    cdef enum Dim2:
        X = 0
        Y = 1

cdef object wrap_vector_double(vector[double] v)
cdef vector[double] make_vector_double(object l)



#!/usr/bin/python

import sys
sys.path.append(os.path.join(os.path.dirname(__file__), "..", "py2geom"))

from  py2geom import *
import py2geom

P = Point(1,2)
Q = Point(3,5)
print P, Q, P+Q

print L2(P)
print cross(P,Q)
#print dir(py2geom)

import numpy

ply = numpy.poly1d([1,0,2])

print ply(0.5)
t = numpy.poly1d([1,0])
q,r = ply / t
print q,r

print ply 
print q*t + r
print ply

def poly_to_sbasis(p):
    sb = SBasis()
    s = numpy.poly1d([-1, 1, 0])
    while True:
        q,r = p / s
        #print "r:", r
        print "r:", repr(r)
        x = Linear(r[0],r[1]+r[0])
        sb.append(x)
        p = q
        print "q:", repr(p), len(list(p))
        if len(list(p)) <= 1 and p[0] == 0:
            return sb

def sbasis_to_poly(sb):
    p = numpy.poly1d([0])
    s = numpy.poly1d([-1, 1, 0])
    sp = numpy.poly1d([1])
    for sbt in sb:
        p += sp*(sbt[0]*(numpy.poly1d([-1,1])) + sbt[1]*(numpy.poly1d([1,0])))
        sp *= s
    return p

trial = numpy.poly1d([1,0,2])
sb = poly_to_sbasis(trial)
print repr(trial),"p2sb:", sb
print "and back again:", repr(sbasis_to_poly(sb))
print repr(sbasis_to_poly(derivative(sb))), repr(trial.deriv())

print "unit tests:"
x = Linear(0,1)
sb = SBasis()
sb.append(x)
print sb
sb = sb*sb
print sb
print sb[0]

print "terms"
for i in range(6):
    sb = SBasis()
    for j in range(3):
        sb.append(Linear(i==2*j,i==2*j+1))
    print sb
    
    print sbasis_to_poly(sb)



#!/usr/bin/python

def choose(n, k):
    r = 1
    for i in range(1, k+1):
        r = (r*(n-k+i))/i
    return r

def W(n, j, k):
    q = (n+1)/2.
    if((n & 1) == 0 and j == q and k == q):
        return 1
    if(k > n-k):
        return W(n, n-j, n-k)
    assert((k <= q))
    if(k >= q):
        return 0
    # assert(!(j >= n-k));
    if(j >= n-k):
        return 0
    # assert(!(j < k));
    if(j < k):
        return 0
    return float(choose(n-2*k-1, j-k)) / choose(n,j);

# this produces a degree 2q bezier from a degree k sbasis
def sbasis_to_bezier(B, q):
    if(q == 0):
        q = len(B)
    n = q*2
    result = [0.0 for i in range(n)]
    if(q > len(B)):
        q = len(B)
    n -= 1
    for k in range(q):
        for j in range(n-k+1):
            result[j] += (W(n, j, k)*B[k][0] +
                          W(n, n-j, k)*B[k][1])
    return result

def lerp(a, b, t):
    return (1-t)*a + t*b



import os
import sys
import argparse

from pygccxml import parser, declarations

class wType:
    def __init__(self, c_str):
        self.c_str = c_str
        self.wrapped_types = ["bint", "double", "int", "unsigned int",
                            "Coord", "IntCoord", "Dim2", "size_t"]

    def get_C(self):
        """returns plain c string"""
        return self.c_str

    def get_Cython_type(self):
        """Returns corresponding Cython type string"""
        typestr= self.get_C()
        typestr= typestr.replace(" const ", " ")
        if typestr.find("::Geom::")==0:
            typestr = typestr[len("::Geom::"):]
        if typestr == "bool":
            typestr = "bint"
        return typestr
        
    def get_Python_function_argument(self):
        """Returns argument type to Python function call"""

        typestr = self.get_Cython_type()
        if typestr[-1]=="&":
            typestr = typestr[:-2]
        if typestr in self.wrapped_types:
            return typestr+" "
        return "cy_"+typestr+" "
    def get_Python_return(self):
        """ Returns a string used to wrap a function returning this type
        returns a string to be called with format( 'function call to c function ')
        """
        typestr = self.get_Cython_type()
        
        if typestr[-1] == "&":
            typestr = typestr[:-2]
        if typestr in self.wrapped_types:
            return "return {}"
        if typestr == "void":
            return "{}"
        if typestr[-1] == "*":
            typestr = typestr[:-2]+"_p"
        return "return wrap_"+typestr+"({})"
        
    def get_wrap_method(self, delim):
        """Returns wrap method in a form of a list of lines"""
        ret = []
        typestr = self.get_Cython_type()
        
        if typestr[-1] == "&":
            typestr = typestr[:-2]
        if typestr in self.wrapped_types:
            return []
        if typestr == "void":
            return []
        if typestr[-1] == "*":
            typestr = typestr[:-2]+"_p"
        ret.append("cdef cy_{} wrap_{}({} p):".format(typestr, typestr, typestr))
        ret.append(delim+"cdef {} * retp = new {}()".format(typestr, typestr))
        ret.append(delim+"retp[0] = p")
        ret.append(delim+"cdef cy_{} r = cy_{}.__new__(cy_{})".format(typestr, typestr, typestr))
        ret.append(delim+"r.thisptr = retp")
        ret.append(delim+"return r")
        return ret
        
    def get_Python_pass_argument(self):
        """ Returns argument to be passed to C function, to be called with .format(arg_name)"""
        typestr = self.get_Cython_type()
        if typestr[-1]=="&":
            typestr = typestr[:-2]
        if typestr in self.wrapped_types:
            return "{}"
        if typestr[-1] == "*":
            return "{}.thisptr"
        return "deref( {}.thisptr )"

class CythonWrapper:
    def __init__(self, gn):
        self.global_namespace = gn
        self.delim = '    '
    
    def wrap_constructor(self, constructor, types_dict):
        pxd_lines = []
        pyx_lines = []
        
        pxd_arguments = []
        for argument_type in constructor.argument_types:
            pxd_arguments.append( types_dict[argument_type.decl_string].get_Cython_type() )
        pxd_argument_str = "({})".format(", ".join(pxd_arguments))
        
        pxd_function_str = constructor.name+pxd_argument_str
        pxd_lines.append(self.delim*2+pxd_function_str)
        
        #python function
        arguments_python = ["self"]
        for argument in constructor.arguments:
            arguments_python.append(types_dict[argument.type.decl_string].get_Python_function_argument()+argument.name)
        declaration = "def {}".format("__init__")+"({}):".format(", ".join(arguments_python))
        
        arguments_cython = []
        for argument in constructor.arguments:
            arguments_cython.append(types_dict[argument.type.decl_string].get_Python_pass_argument().format(argument.name))
        function_call = "new "+constructor.name+"({})".format(", ".join(arguments_cython))
        
        return_statement = "self.thisptr = {}".format(function_call)
        pyx_lines.append(self.delim+declaration)
        pyx_lines.append(self.delim*2 + return_statement)
        
        return (pxd_lines, pyx_lines)

    def wrap_operator(self, operator, types_dict):
        pxd_lines = []
        pyx_lines = []

        pxd_arguments = []
        for argument_type in operator.argument_types:
            pxd_arguments.append( types_dict[argument_type.decl_string].get_Cython_type() )
        pxd_argument_str = "({})".format(", ".join(pxd_arguments))
        boost_wrapped = ["operator{}=".format(i) for i in ["+", "-", "*", "/", "|", "&"]]
        pxd_function_str = types_dict[operator.return_type.decl_string].get_Cython_type()+\
                            " "+operator.name+pxd_argument_str
        
        if not (operator.name in boost_wrapped):
            pxd_lines.append(self.delim*2+pxd_function_str)

        else:
            pxd_lines.append(self.delim*2+"#"+pxd_function_str)

            pxd_function_str = types_dict[operator.return_type.decl_string].get_Cython_type()+\
            " "+operator.name[:-1]+pxd_argument_str
            pxd_lines.append(self.delim*2+pxd_function_str)


        special_methods = {
            "+" : "__add__",
            "-" : "__sub__",
            "*" : "__mul__",
            "/" : "__div__",
            "|" : "__or__",
            "&" : "__and__",
            "[]": "__getitem__",
            "()": "__call__",
            "==": "__richcmp__",
            "!=": "__richcmp__"
        }
        
        if operator.name in boost_wrapped:
            operation = operator.name[-2]
        else:
            operation = operator.name[ len("operator"): ]
            
        if operation in special_methods:
            python_name = special_methods[operation]
        else:
            python_name = operator.name
        
        arguments_python = ["self"]
        for argument in operator.arguments:
            arguments_python.append(types_dict[argument.type.decl_string].get_Python_function_argument()+argument.name)
        declaration = "def {}".format(python_name)+"({}):".format(", ".join(arguments_python))
        
        arguments_cython = []
        for argument in operator.arguments:
            arguments_cython.append(types_dict[argument.type.decl_string].get_Python_pass_argument().format(argument.name))
        function_call = "deref( self.thisptr ) "+operation+" {}".format(", ".join(arguments_cython))
        
        return_statement = types_dict[operator.return_type.decl_string].get_Python_return().format(function_call)
        
        if (operator.arguments == []) and (operation == '-'):
            declaration = "def __neg__(self):"
        pyx_lines.append(self.delim+declaration)
        pyx_lines.append(self.delim*2 + return_statement)

        return (pxd_lines, pyx_lines)

    def wrap_method(self, method, types_dict):
        pxd_lines = []
        pyx_lines = []
        
        pxd_arguments = []
        for argument_type in method.argument_types:
            pxd_arguments.append( types_dict[argument_type.decl_string].get_Cython_type() )
        pxd_argument_str = "({})".format(", ".join(pxd_arguments))
        
        
        pxd_function_str = types_dict[method.return_type.decl_string].get_Cython_type()+\
                            " "+method.name+pxd_argument_str
        pxd_lines.append(self.delim*2+pxd_function_str)
        
        #python function
        arguments_python = ["self"]
        for argument in method.arguments:
            arguments_python.append(types_dict[argument.type.decl_string].get_Python_function_argument()+argument.name)
        declaration = "def {}".format(method.name)+"({}):".format(", ".join(arguments_python))
        
        arguments_cython = []   
        for argument in method.arguments:
            arguments_cython.append(types_dict[argument.type.decl_string].get_Python_pass_argument().format(argument.name))
        function_call = "self.thisptr."+method.name+"({})".format(", ".join(arguments_cython))
        
        return_statement = types_dict[method.return_type.decl_string].get_Python_return().format(function_call)
        pyx_lines.append(self.delim+declaration)
        pyx_lines.append(self.delim*2 + return_statement)
        
        return (pxd_lines, pyx_lines)
    
    def wrap_free_function(self, function, types_dict):
        pxd_lines = []
        pyx_lines = []
        decl_lines = []
                                
        pxd_arguments = []
        for argument_type in function.argument_types:
            pxd_arguments.append( types_dict[argument_type.decl_string].get_Cython_type() )
        pxd_argument_str = "({})".format(", ".join(pxd_arguments))
        
        
        pxd_function_str = types_dict[function.return_type.decl_string].get_Cython_type()+\
                            " "+function.name+pxd_argument_str
        pxd_lines.append(self.delim+pxd_function_str)
        
        #python function
        arguments_python = []
        for argument in function.arguments:
            arguments_python.append(types_dict[argument.type.decl_string].get_Python_function_argument()+argument.name)
        declaration = "def cy_{}".format(function.name)+"({}):".format(", ".join(arguments_python))
        
        decl_lines.append("from _cy_??? import cy_{0} as {0}".format(function.name))
        
        arguments_cython = []   
        for argument in function.arguments:

            arguments_cython.append(types_dict[argument.type.decl_string].get_Python_pass_argument().format(argument.name))
        function_call = function.name+"({})".format(", ".join(arguments_cython))
        
        return_statement = types_dict[function.return_type.decl_string].get_Python_return().format(function_call)
        pyx_lines.append(declaration)
        pyx_lines.append(self.delim + return_statement)
        
        return (pxd_lines, pyx_lines, decl_lines)

    
    def wrap_class(self, class_name, class_file):
        pxd_lines = []
        pyx_lines = []
        decl_lines = []
        other_lines = []

        pxd_lines.append("cdef extern from \"2geom/{}\" namespace \"Geom\":".format(class_file))
        pxd_lines.append(self.delim+"cdef cppclass {}:".format(class_name))
        pyx_lines.append("cdef class cy_{}:".format(class_name))
        pyx_lines.append(self.delim+"cdef {}* thisptr".format(class_name))
        #get_types
        types_dict = self.collect_types(class_name)
        #parse
        c_class = self.global_namespace.namespace(name = "Geom").class_(name=class_name)

        for member in c_class.get_members():
            if isinstance(member, declarations.calldef.constructor_t):
                pxd, pyx = self.wrap_constructor(member, types_dict)
                pxd_lines += pxd
                pyx_lines += pyx
            elif isinstance(member, declarations.calldef.member_operator_t):
                pxd, pyx = self.wrap_operator(member, types_dict)
                pxd_lines += pxd
                pyx_lines += pyx
            elif isinstance(member, declarations.calldef.member_function_t):
                pxd, pyx = self.wrap_method(member, types_dict)
                pxd_lines += pxd
                pyx_lines += pyx
                
        pyx_lines.append("#free functions:")
        for ff in self.global_namespace.namespace(name = "Geom").free_funs():
            if os.path.basename(ff.location.file_name) == class_file:
                pxd, pyx, decl = self.wrap_free_function(ff, types_dict)
                pxd_lines += map(lambda x: x[4:], pxd)
                pyx_lines += pyx
                decl_lines += decl
                
        for ff in self.global_namespace.namespace(name = "Geom").free_operators():
            if os.path.basename(ff.location.file_name) == class_file:
                pxd, pyx = self.wrap_operator(ff, types_dict)
                pxd_lines += map(lambda x: x[4:], pxd)
                pyx_lines += pyx
   
        pyx_lines.append("")
        for ctype in types_dict:
            wtype = types_dict[ctype]
            for line in wtype.get_wrap_method(self.delim):
                pyx_lines.append(line)
        print "------"
        for i in pxd_lines:
            print i
        print "------"
        for i in pyx_lines:
            print i
        print "------"
        for i in decl_lines:
            print i
            
           
    def collect_types(self, class_name):
        types_set = set()
        c_class = self.global_namespace.namespace(name = "Geom").class_(name=class_name)
        for member in c_class.get_members():
            if any([isinstance(member, declarations.calldef.constructor_t),
                    isinstance(member, declarations.calldef.member_operator_t),
                    isinstance(member, declarations.calldef.member_function_t)]):
                for a_type in member.argument_types:
                    types_set.add(a_type.decl_string) 
                if isinstance(member, declarations.calldef.member_operator_t) or isinstance(member, declarations.calldef.member_function_t):
                    types_set.add(member.return_type.decl_string)
        for ff in list(self.global_namespace.namespace(name = "Geom").free_funs()) + list(self.global_namespace.namespace(name = "Geom").free_operators()):
            for a_type in ff.argument_types:
                types_set.add(a_type.decl_string)
                types_set.add(ff.return_type.decl_string)
        types_dict = dict()
        
        for f_type in types_set:

            types_dict[f_type] = wType(f_type)
        return types_dict

def main():

    #set up argument parser
    cmd_parser = argparse.ArgumentParser()
    cmd_parser.add_argument('--lib2geom_dir', action = 'store')
    cmd_parser.add_argument('--file_name', action = 'store')
    cmd_parser.add_argument('--class_name', action = 'store')
    cmd_args = cmd_parser.parse_args()
    
    #set up pygccxml
    includes = [cmd_args.lib2geom_dir, 
                os.path.join(cmd_args.lib2geom_dir, 'src'), 
                "/usr/include/boost"]

    config = parser.config_t(compiler='gcc', 
                            include_paths=includes, 
                            cflags="")
    
    file_to_parse = [os.path.join(cmd_args.lib2geom_dir, "src", "2geom", cmd_args.file_name)] 
    
    decls = parser.parse( file_to_parse, config )
    global_namespace = declarations.get_global_namespace( decls )
    wrapper = CythonWrapper(global_namespace)
    wrapper.wrap_class(cmd_args.class_name, cmd_args.file_name)

main()

#run with
#python2 wrapper.py --lib2geom_dir ../../.. --file_name int-point.h --class_name IntPoint
#from cython-bindings directory



#!/usr/bin/python

import py2geom
import toyframework
import random,gtk
from py2geom_glue import *

def cairo_region(cr, r):
    cr.save()
    cr.set_source_rgba(0, 0, 0, 1);
    if not r.isFill():
        pass#cr.set_dash([]1, 0)
    cairo_path(cr, r)
    cr.fill()
    cr.restore()

def cairo_regions(cr, p):
    for j in p:
        cairo_region(cr, j)

def cairo_shape(cr, s):
    cairo_regions(cr, s.getContent())



def cleanup(ps):
    pw = py2geom.paths_to_pw(ps)
    centre, area = py2geom.centroid(pw)
    if(area > 1):
        return py2geom.sanitize(ps) * py2geom.Translate(-centre)
    else:
        return py2geom.sanitize(ps)

class LoadSVGD(toyframework.Toy):
    def __init__(self):
        toyframework.Toy.__init__(self)
        self.bs = []
        self.offset_handle = toyframework.PointHandle(0,0)
        self.handles.append(self.offset_handle)
    def draw(self, cr, pos, save):
        t = py2geom.Translate(*self.offset_handle.pos)
        #self.paths_b[0] * t
        m = py2geom.Matrix()
        m.setIdentity()
        bst = self.bs * (m * t)
        #bt = Region(b * t, b.isFill())
        
        cr.set_line_width(1)
        
        cairo_shape(cr, bst)
        
        toyframework.Toy.draw(self, cr, pos, save)
        
    def first_time(self, argv):
        path_b_name="star.svgd"
        if len(argv) > 1:
            path_b_name = argv[1]
        self.paths_b = py2geom.read_svgd(path_b_name)
        
	bounds = py2geom.bounds_exact(self.paths_b)
        self.offset_handle.pos = bounds.midpoint() - bounds.corner(0)

        self.bs = cleanup(self.paths_b)
t = LoadSVGD()
import sys

toyframework.init(sys.argv, t, 500, 500)




#!/usr/bin/python

import gtk,gtk.gdk,math,pango  #Numeric,

templayout = None

def draw_spot(w, h):
    x,y = h.x, h.y
    g = gtk.gdk.GC(w)
    w.draw_line(g, int(x), int(y), int(x), int(y))

def draw_handle(w, h, name = ""):
    x,y = h.x, h.y
    g = gtk.gdk.GC(w)
    w.draw_line(g, int(x-3), int(y), int(x+3), int(y))
    w.draw_line(g, int(x), int(y-3), int(x), int(y+3))
    templayout.set_text(name)
    w.draw_layout(g, x, y, templayout)

def draw_ray(w, h, d):
    x,y = h.x, h.y
    g = gtk.gdk.GC(w)
    w.draw_line(g, int(h.x), int(h.y), int(3*d.x-2*h.x), int(3*d.y-2*h.y))

def intersect(n0, d0, n1, d1):
    denominator = n0.x*n1.y - n0.y*n1.x
    X = n1.y * d0 - n0.y * d1
    if denominator == 0:
        return None;

    Y = n0.x * d1 - n1.x * d0

    return handle(X / denominator, Y / denominator)

def seg(a0, a1, b0, b1):
    n0 = handle(a1.y - a0.y, -a1.x + a0.x)
    d0 = n0.x*a0.x + n0.y *a0.y
    n1 = handle(b1.y - b0.y, -b1.x + b0.x)
    d1 = n1.x*b0.x + n1.y *b0.y

    return intersect(n0, d0, n1, d1)

def draw_elip(w, h):
    g = gtk.gdk.GC(w)
    w.draw_line(g, h[0].x, h[0].y, h[1].x, h[1].y)
    w.draw_line(g, h[3].x, h[3].y, h[4].x, h[4].y)
    w.draw_line(g, h[3].x, h[3].y, h[2].x, h[2].y)
    w.draw_line(g, h[2].x, h[2].y, h[1].x, h[1].y)

    c = seg(h[0], h[1], h[3], h[4])
    draw_handle(w, c)
    
    if 0:
        for i in range(6):
            w.draw_line(g, h[i].x, h[i].y, h[(i+1)%6].x, h[(i+1)%6].y)
        
    
    cx,cy = c.x, c.y

    ox, oy = None, None
    for i in range(0, 101):
        t = i/100.0
        
        
        nx = (1-t)*h[0].x + t*h[3].x
        ny = (1-t)*h[0].y + t*h[3].y
        #w.draw_line(g, 2*cx-nx, 2*cy-ny, nx, ny)
        c1 = seg(handle(2*cx-nx, 2*cy-ny), handle(nx, ny), h[0], h[2])
        #draw_handle(w, c1)
        c2 = seg(handle(2*cx-nx, 2*cy-ny), handle(nx, ny), h[4], h[2])
        #draw_handle(w, c2)
        #draw_ray(w, h[3], c1)
        #draw_ray(w, h[1], c2)
        six = seg(c1, h[3], c2, h[1])
        #draw_spot(w, six)
        if ox:
            w.draw_line(g, ox, oy, six.x, six.y)
        ox, oy = six.x, six.y
    return
    
    r = math.hypot(h[0].x - cx, h[0].y - cy)
    s = math.atan2(h[0].y - h[3].y, h[0].x - h[3].x)
    e = math.atan2(h[1].y - h[4].y, h[1].x - h[4].x)
    for i in range(0, 101):
        t = (e-s)*i/100.0 + s
        nx, ny = r*math.cos(t)+cx, r*math.sin(t)+cy
        sx, sy = r*math.cos(t+math.pi)+cx, r*math.sin(t+math.pi)+cy
        w.draw_line(g, sx, sy, nx, ny)
    

class handle:
    def __init__(self, x, y):
        self.x = x
        self.y = y
    def __repr__(self):
        return "handle(%f, %f)" % (self.x, self.y)

handles = [handle(145.000000, 50.000000), handle(43.000000, 69.000000), handle(26.000000, 135.000000), handle(48.000000, 189.000000), handle(248.000000, 188.000000)]

selected_handle = None

def display(da, ev):
    g = gtk.gdk.GC(da.window)
    i = 0

    
    
    for h in handles:
        draw_handle(da.window, h, str(i))
        i += 1
    draw_elip(da.window, handles)

def mouse_event(w, e):
    global selected_handle
    if e.button == 1:
        for h in handles:
            if math.hypot(e.x-h.x, e.y - h.y) < 5:
                selected_handle = (h, (e.x-h.x, e.y-h.y))

def mouse_release_event(w, e):
    global selected_handle
    selected_handle = None

def mouse_motion_event(w, e):
    global selected_handle
    if selected_handle:
        h, (ox, oy) = selected_handle
        if(e.state & gtk.gdk.BUTTON1_MASK):
            h.x = e.x - ox
            h.y = e.y - oy
            w.queue_draw()


win = gtk.Window()
win.set_default_size(400,400)
vb = gtk.VBox(False)

da = gtk.DrawingArea()
templayout = da.create_pango_layout("")
da.connect("expose_event", display)
da.add_events(gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.BUTTON_RELEASE_MASK | gtk.gdk.KEY_PRESS_MASK | gtk.gdk.POINTER_MOTION_MASK)
da.connect("button-press-event", mouse_event)
da.connect("button-release-event", mouse_release_event)
da.connect("motion-notify-event", mouse_motion_event)
#da.connect("key_press_event", key_event)
win.add(vb)
vb.pack_start(da)
win.connect("destroy", gtk.main_quit)


win.show_all()

gtk.main()

print handles



# test of 2geom ray bindings

import py2geom as g

# find one point along a ray
a = g.Point(0,0)
b = g.Point(2,2)

r = g.Ray(a,b)
from math import sqrt
print r.pointAt(sqrt(2))

# measure the angle between two rays
c = g.Point(2,-2)
r2 = g.Ray(a,c)
from math import degrees
# FIXME: the third argument (clockwise) ought to be optional, but has to be supplied
print degrees(g.angle_between(r, r2, True))
print degrees(g.angle_between(r, r2))




#!/usr/bin/python

import py2geom
import toyframework
import random,gtk
import numpy
from py2geom_glue import *

class Box3d(toyframework.Toy):
    def __init__(self):
        toyframework.Toy.__init__(self)
        self.tmat = numpy.zeros([3,4])
        # plane origin
        self.origin_handle = toyframework.PointHandle(180,65)
        self.handles.append(self.origin_handle)
        self.vanishing_points_handles = toyframework.PointSetHandle()
        path_a_name="ptitle.svgd"
        import sys
        if len(sys.argv) > 1:
            path_a_name = sys.argv[1]
        self.paths_a = py2geom.read_svgd(path_a_name)
    

        # Finite images of the three vanishing points and the origin
        self.handles.append(self.vanishing_points_handles)
        self.vanishing_points_handles.append(550,350)
        self.vanishing_points_handles.append(150,300)
        self.vanishing_points_handles.append(380,40)
        self.vanishing_points_handles.append(340,450)
    def draw(self, cr, pos, save):
        orig = self.origin_handle.pos;
	cr.set_source_rgba (0., 0.125, 0, 1)

        # create the transformation matrix for the map  P^3 --> P^2 that has the following effect:
        #      (1 : 0 : 0 : 0) --> vanishing point in x direction (= handle #0)
        #      (0 : 1 : 0 : 0) --> vanishing point in y direction (= handle #1)
        #      (0 : 0 : 1 : 0) --> vanishing point in z direction (= handle #2)
        #      (0 : 0 : 0 : 1) --> origin (= handle #3)
        
        tmat = numpy.zeros([3,4])
        for j in range(4):
            tmat[0][j] = self.vanishing_points_handles.pts[j][0]
            tmat[1][j] = self.vanishing_points_handles.pts[j][1]
            tmat[2][j] = 1

        self.notify = "Projection matrix:\n"
        for i in range(3):
            for j in range(4):
                self.notify += str(tmat[i][j]) + " "
            self.notify += '\n'

        for p in self.paths_a:
            B = py2geom.make_cuts_independant(p.toPwSb())
            preimage = [None]*4
                
            preimage[0] =  (B[0] - orig[0]) / 100;
            preimage[1] = -(B[1] - orig[1]) / 100;
            Piecewise<SBasis> res[3];
            for j in range(3):
                res[j] = (preimage[0] * tmat[j][0] +
                          preimage[1] * tmat[j][1] +
                          + tmat[j][3])
            
            result = D2PiecewiseSBasis(divide(res[0],res[2], 2), 
                                       divide(res[1],res[2], 2))
            
            toyframework.cairo_d2_pw(cr, result)
            cr.set_source_rgba (0., 0.125, 0, 1)
            cr.stroke()
        
        toyframework.Toy.draw(self, cr, pos, save)
    
t = Box3d()
import sys

toyframework.init(sys.argv, t, 500, 500)





import math

def q(x, a, b, c):
    sa = math.sqrt(a)
    y = math.sqrt(a*x*x+b*x+c)
    dp = 2*a*x + b
    r = dp*y/(4*a)
    t = abs(dp + 2*y*sa)
    s = (4*a*c-b*b)/(a*sa*8)
    return r+s*math.log(t)

a = 1160390
b = -922658
c = 249477
print q(1, a,b,c) - q(0,a,b,c)




#!/usr/bin/python

from Numeric import *
from LinearAlgebra import *

pascals_triangle = []
rows_done = 0

def choose(n, k):
    r = 1
    for i in range(1,k+1):
        r *= n-k+i
        r /= i
    return r

# http://www.research.att.com/~njas/sequences/A109954
def T(n, k):
    return ((-1)**(n+k))*choose(n+k+2, 2*k+2)

def inver(q):
    result = zeros((q+2,q+2))
    q2 = q/2+1
    for i in range(q2):
        for j in range(i+1):
            val = T(i,j)
            result[q/2-j][q/2-i] = val
            result[q/2+j+2][q/2+i+2] = val
            result[q/2+j+2][q/2-i-1] = -val
            if q/2+i+3 < q+2:
                result[q/2-j][q/2+i+3] = -val
            
    for i in range(q+2):
        result[q2][i] = [1,-1][(i-q2)%2]
    return result
        
def simple(q):
    result = zeros((q+2,q+2))
    for i in range(q/2+1):
        for j in range(q+1):
            result[j][i] = choose(q-2*i, j-i)
            result[j+1][q-i+1] = choose(q-2*i, j-i)
    result[q/2+1][q/2+1] = 1
    return result

print "The aim of the game is to work out the correct indexing to make the two matrices match :)"

s = simple(4)
si = floor(inverse(s)+0.5)
print si.astype(Int)
print inver(4)
exit(0)
print "<html><head><title></title></head><body>"

def arrayhtml(a):
    s = "<table>"
    r,c = a.shape
    for i in range(r):
        s += "<tr>";
        for j in range(c):
            s += "<td>%g</td>" % a[i,j]
        s += "</tr>"
    s += "</table>"
    return s

for i in [21]:#range(1,13,2):
    s = simple(i)
    print "<h1>T<sup>-1</sup> = </h1>"
    print arrayhtml(s)
    print "<h1>T = </h1>"
    print arrayhtml(floor(inverse(s)+0.5))

print "</body></html>"



from _common_decl cimport *

from libcpp.vector cimport vector
from libcpp.pair cimport pair

from _cy_rectangle cimport Interval, OptInterval, Rect, OptRect
from _cy_rectangle cimport cy_OptRect
from _cy_affine cimport is_transform, get_Affine, Affine
from _cy_curves cimport Curve, cy_Curve,  wrap_Curve_p
from _cy_curves cimport SBasis, cy_SBasis
from _cy_curves cimport EllipticalArc, cy_EllipticalArc, wrap_EllipticalArc

from _cy_path cimport Path, cy_Path

from _cy_primitives cimport Point, cy_Point, wrap_Point, wrap_vector_point, make_vector_point


cdef extern from "2geom/circle.h" namespace "Geom":
    cdef cppclass Circle:
        Circle()
        Circle(double, double, double)
        Circle(Point, double)
        Circle(double, double, double, double)
        Circle(vector[Point] &)
        void setCenter(Point &)
        void setRadius(double)
        void set(double, double, double, double)
        void fit(vector[Point] &)
        EllipticalArc * arc(Point &, Point &, Point &, bint)
        Point center()
        Coord center(Dim2)
        Coord radius()

cdef extern from "2geom/ellipse.h" namespace "Geom":
    cdef cppclass Ellipse:
        Ellipse()
        Ellipse(double, double, double, double, double)
        Ellipse(double, double, double, double, double, double)
        Ellipse(vector[Point] &)
        Ellipse(Circle &)
        void set(double, double, double, double, double)
        void set(double, double, double, double, double, double)
        void set(vector[Point] &)
        EllipticalArc * arc(Point &, Point &, Point &, bint)
        Point center()
        Coord center(Dim2)
        Coord ray(Dim2)
        Coord rot_angle()
        vector[double] implicit_form_coefficients()
        Ellipse transformed(Affine &)



# Copyright 2006, 2007 Aaron Spike <aaron@ekips.org>
#
# This library is free software; you can redistribute it and/or
# modify it either under the terms of the GNU Lesser General Public
# License version 2.1 as published by the Free Software Foundation
# (the "LGPL") or, at your option, under the terms of the Mozilla
# Public License Version 1.1 (the "MPL"). If you do not alter this
# notice, a recipient may use your version of this file under either
# the MPL or the LGPL.
#
# You should have received a copy of the LGPL along with this library
# in the file COPYING-LGPL-2.1; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
# You should have received a copy of the MPL along with this library
# in the file COPYING-MPL-1.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# This software is distributed on an "AS IS" basis, WITHOUT WARRANTY
# OF ANY KIND, either express or implied. See the LGPL or the MPL for
# the specific language governing rights and limitations.

from py2geom._py2geom import *



import unittest
import math
from random import randint, uniform

import cy2geom

from cy2geom import Point, IntPoint
from cy2geom import Interval, IntInterval, OptInterval, OptIntInterval

from cy2geom import Affine
from cy2geom import Translate, Scale, Rotate, VShear, HShear, Zoom
from cy2geom import Eigen

from cy2geom import Curve
from cy2geom import Linear
from cy2geom import SBasis, SBasisCurve
from cy2geom import Bezier, BezierCurve

from cy2geom import LineSegment, QuadraticBezier, CubicBezier
from cy2geom import HLineSegment, VLineSegment

from cy2geom import EllipticalArc

from cy2geom import Path

from cy2geom import Circle, Ellipse


class TestPrimitives(unittest.TestCase):
    def test_circle(self):
        C = Circle()
        self.assertEqual(C.center(), Point())
        
        D = Circle(Point(2, 4), 2)
        Dp = D.getPath()
        self.assertEqual(D.center(), Point(2, 4))
        self.assertEqual(D.ray(), 2)

        for i in range(11):
            t = i/10.0
            #Circle approximated by SBasis is not perfect
            self.assertAlmostEqual( abs(D.center()-Dp(t)), D.ray(), delta=0.1 )

        half_circle = D.arc(Dp(0), Dp(0.3), Dp(0.5))

        self.assertTrue(half_circle.is_SVG_compliant())

        self.assertAlmostEqual(Dp(0.25), half_circle(0.5), delta=0.1)

        points = [Point(2, 5), Point(1, 4), Point(9, 0)]
        D.set_points(points)
        for p in points:
            self.assertAlmostEqual( abs(p-D.center()), D.ray() )
        Dc = Circle.from_points(points)
        self.assertAlmostEqual(Dc.center(), D.center())
        self.assertAlmostEqual(Dc.ray(), D.ray())
        
        coeffs = (2, 4, 1, -4)
        E = Circle.from_coefficients(*coeffs)
        def param(x, y):
            A, B, C, D = coeffs
            return A*x**2 + A*y**2 + B*x + C*y + D
        Ec = E.arc(E.center()+Point(E.ray(), 0), E.center()-Point(E.ray(), 0), E.center()+Point(E.ray(), 0) )
        for i in range(11):
            t = i/10.0
            self.assertAlmostEqual(param(Ec.value_at(t, 0), Ec.value_at(t, 1)), 0)
        
        E.set(3, 5, 9)
        self.assertAlmostEqual(E.center(), Point(3, 5))
        self.assertAlmostEqual(E.ray(), 9)
        
        E.set_coefficients(*coeffs)
        #radius and center from parametric equation 
        ca = float(coeffs[1])/coeffs[0]
        cb = float(coeffs[2])/coeffs[0]
        cc = float(coeffs[3])/coeffs[0]
        self.assertAlmostEqual( 4*E.ray()**2 , ca**2 + cb**2 -4*cc )
        self.assertAlmostEqual( E.center(), -Point(ca, cb)/2)
    
    def test_ellipse(self):
        #TODO: maybe a bug in arc? get_curve(F) returns different ellipse than F
        def get_curve(ellipse):
            p = Point(ellipse.ray(0), 0)*Rotate(ellipse.rot_angle())
            return ellipse.arc(ellipse.center()+p, ellipse.center()-p, ellipse.center()+p*(1-1e-7))
        E = Ellipse()
        self.assertAlmostEqual(E.center(), Point())
        self.assertAlmostEqual(E.ray(0), 0)
        self.assertAlmostEqual(E.ray(1), 0)
        
        F = Ellipse(Point(), 3, 2, 0)
        self.assertAlmostEqual(F.center(), Point())
        self.assertAlmostEqual(F.ray(0), 3)
        self.assertAlmostEqual(F.ray(1), 2)
        self.assertAlmostEqual(F.rot_angle(), 0)
        # x**2/9 + y**2/4 = 1
        self.assertAlmostEqual(F.implicit_form_coefficients()[0], 1/9.0)
        self.assertAlmostEqual(F.implicit_form_coefficients()[2], 1/4.0)
        self.assertAlmostEqual(F.implicit_form_coefficients()[5], -1)
        
        coeffs = (1/3.0, 0, 1/16.0, 1, 0, -1/4.0)
        G = Ellipse.from_coefficients(*coeffs)
        self.assertAlmostEqual(G.center(), Point(-3/2.0, 0))
        self.assertAlmostEqual(G.ray(0), math.sqrt(3))
        self.assertAlmostEqual(G.ray(1), 4)
        self.assertAlmostEqual(G.rot_angle(), 0)
        
        points = [Point(1, 2), Point(2 ,9), Point(0, 3), Point(-3, 8), Point(5, 8)]
        G.set_points(points)
        coeffs_G = tuple(G.implicit_form_coefficients())
        def paramG(x, y):
            A, B, C, D, E, F = coeffs_G
            return A*x**2 + B*x*y + C*y**2 + D*x + E*y + F
        for p in points:
            self.assertAlmostEqual(paramG(p.x, p.y), 0)

        G2 = Ellipse.from_points(points)
        coeffs_G2 = tuple(G.implicit_form_coefficients())
        def paramG2(x, y):
            A, B, C, D, E, F = coeffs_G2
            return A*x**2 + B*x*y + C*y**2 + D*x + E*y + F
        for p in points:
            self.assertAlmostEqual(paramG2(p.x, p.y), 0)
        
        E.set_coefficients(*coeffs_G2)
        for a1, a2 in zip(E.implicit_form_coefficients(), G2.implicit_form_coefficients()):
            self.assertAlmostEqual(a1, a2)

        H = Ellipse.from_circle(Circle(Point(2, 8), 5))
        self.assertAlmostEqual(H.center(), Point(2, 8))
        self.assertAlmostEqual(H.ray(0), 5)
        self.assertAlmostEqual(H.ray(1), 5)
        
        Ft = F.transformed( Rotate(math.pi/2) )
        self.assertAlmostEqual(F.ray(0), Ft.ray(1))
        self.assertAlmostEqual(F.ray(1), Ft.ray(0))
        
unittest.main()



import unittest
from math import pi, sqrt
from random import randint

import cy2geom

from cy2geom import Point, IntPoint

from cy2geom import Interval, IntInterval, OptInterval, OptIntInterval
from cy2geom import GenericInterval, GenericOptInterval

from cy2geom import Rect, OptRect, IntRect, OptIntRect
from cy2geom import GenericRect

from fractions import Fraction
        

class TestPrimitives(unittest.TestCase):
    def interval_basic(self, I, J):
        #for simplicity
        self.assertTrue(I.min() >= 0)
        self.assertTrue(J.min() >= 0)
        a = I.min()
        b = I.max();
        self.assertAlmostEqual(I.middle(), (a+b)/2); 
        self.assertAlmostEqual(I.extent(), (b-a)); 
        if a != b:
            self.assertFalse(I.is_singular())
        else:
            self.assertTrue(I.is_singular())
            
        I.expand_by(a)
        self.assertAlmostEqual(I.min(), 0); 
        self.assertAlmostEqual(I.max(), a+b)
        I.set_min(a)
        I.set_max(b)
        
        self.assertTrue(I.contains(a+ (b-a)/3 ))
        self.assertTrue(I.contains(a))
        self.assertTrue(I.contains_interval(I))
        
        if (not I.is_singular()) or I.min() != 0 :
            pass
        self.assertFalse(I.contains_interval(I+I))
        self.assertFalse(I.contains(a-1))
        
        c = J.min()
        d = J.max()
        self.assertAlmostEqual( (I+J).min(), a+c )
        self.assertAlmostEqual((I|J).min(), min(a, c))
        J.set_min(a+2)
        J.set_max(b+2)
        self.assertEqual(I+2, J)
        I.expand_to(2*b)
        self.assertAlmostEqual(I.max(), 2*b)
        
    def test_interval(self):
        I = Interval(1.2, 5)
        J = Interval(0, 0.3)
        self.interval_basic(I, J)
        
        self.assertTrue(I.interior_contains(I.middle()))
        self.assertFalse(I.interior_contains(I.min()))
        self.assertFalse(I.interior_contains_interval(I))
        self.assertTrue(I.interior_contains_interval(Interval(I.min()+1, I.max()-1)))
        
        self.assertTrue(I.interior_intersects(I))
        self.assertFalse(I.interior_intersects(-I))
        p = [1, 2, 3.442, 3]
        K = Interval.from_list(p)
        self.assertAlmostEqual(K.max(), max(p))
        self.assertAlmostEqual((K+Interval(1.0)).min(), min(p)+1)
        L = Interval(10/3.0)
        for i in range(3):
            K+=L
        self.assertAlmostEqual(K.max(), max(p)+10)
        
        #TODO This 2geom behaviour is a bit strange
        self.assertEqual(Interval(3.0)|Interval(5.0), 
                    Interval(3.0, 5.0))
        self.assertAlmostEqual((K-L).max(), (K-10/3.0).max())
        
        self.assertAlmostEqual((K*3.4).max(), 3.4*K.max())
        self.assertAlmostEqual((K/3).extent(), K.extent()/3)
        
    def test_optInterval(self):
        I = OptInterval(2.2, 9.3)
        J = Interval(3, 13)
        K = OptInterval.from_Interval(J)
        self.assertEqual(K.Interval, J)
        self.interval_basic(K.Interval, I.Interval)
        
        L = OptInterval()
        
        self.assertFalse(L)
        self.assertTrue( (L&I).is_empty() )
        L.intersect_with(I)
        self.assertFalse(L)
        
        L |= I
        
        self.assertEqual(L.Interval, I.Interval)
        
        self.assertEqual((I & K).Interval, Interval(3, 9.3))
        
    def test_intInterval(self):
        I = IntInterval(2, 6)
        J = IntInterval(0, 1)
        self.interval_basic(I, J)
        p = [3, 2.3, 65.3, 43]
        K = IntInterval.from_list(p)
        self.assertAlmostEqual(K.max(), int(max(p)))
        self.assertAlmostEqual(int((K+IntInterval(1.0)).min()), int(min(p)+1))
        L = IntInterval(3)
        for i in range(3):
            K+=L
        self.assertAlmostEqual(K.max(), int(max(p))+9)
        
        self.assertEqual(Interval(3)|Interval(5), 
                    Interval(3, 5))
        self.assertAlmostEqual((K-L).max(), (K-3).max())

    def test_optIntInterval(self):
        I = OptIntInterval(2, 9)
        J = IntInterval(3, 13)
        K = OptIntInterval.from_Interval(J)
        self.assertEqual(K.Interval, J)
        self.interval_basic(K.Interval, I.Interval)
        
        L = OptIntInterval()
        
        self.assertFalse(L)
        self.assertTrue( (L&I).is_empty() )
        L.intersect_with(I)
        self.assertFalse(L)
        
        L |= I
        
        self.assertEqual(L.Interval, I.Interval)
        
        self.assertEqual((I & K).Interval, IntInterval(3, 9))
        
    def test_genericInterval(self):
        maxv = 100000
        test_vars = [
            ( (randint(0, maxv), randint(0, maxv)), (randint(0, maxv), randint(0, maxv)) ),
            ( (3,), (2, 0) ),
            ( (0.0, 9), (4, 1.3)),
            ((2.98, sqrt(2)), (sqrt(7),)),
            ( (Fraction(1,2), Fraction(3, 7)), ( Fraction(2, 1), ) )
            ]
        for a,b in test_vars:
            self.interval_basic( GenericInterval(*a), GenericInterval(*b) )
            
    def test_genericOptInterval(self):
        test_vars = [
            ( (3,), (2, 0) ),
            ( (0.0, 9), (4, 1.3)),
            ((2.98, sqrt(2)), (sqrt(7),)),
            ( (Fraction(1,2), Fraction(3, 7)), ( Fraction(2, 1), ) )
            ]
            
        for a, b in test_vars:
            I = GenericOptInterval(*a)
            J = GenericInterval(*b)
            K = GenericOptInterval.from_Interval(J)
            
            self.assertEqual(I, GenericOptInterval.from_Interval(I))
            
            self.assertEqual(K.Interval, J)
            self.interval_basic(K.Interval, I.Interval)
            
            L = GenericOptInterval()
            
            self.assertFalse(L)
            self.assertTrue( (L&I).is_empty() )
            L.intersect_with(I)
            self.assertFalse(L)
            
            L |= I
            
            self.assertEqual(L.Interval, I.Interval)
            
            if I.intersect_with(K):
                if I.Interval.min() <= K.Interval.min():
                    if I.Interval.max() >= K.Interval.max():
                        self.assertEqual( I & K, K)
                    else:
                        self.assertEqual( I & K, GenericInterval(K.min(), I.max()))
                else:
                    if I.Interval.max() >= K.Interval.max():
                        self.assertEqual( I & K, GenericInterval(I.min(), K.max()))
                    else:
                        self.assertEqual( I & K, I)
    
    def test_genericRect(self):
        A = GenericRect(1, 1, 4, 4)
        self.assertEqual( A.min(), (1, 1) )
        B = GenericRect(Fraction(1,4), Fraction(9, 94), Fraction(2, 3), Fraction(23, 37))

        amin = A.min()
        amax = A.max()
        
        self.assertAlmostEqual(amin[0], A[0].min())
        self.assertAlmostEqual(amax[1], A[1].max())
        self.assertEqual(amin, A.corner(0))
        
        self.assertEqual(amin, (A.left(), A.top()))
        self.assertEqual(amax, (A.right(), A.bottom()))
        
        self.assertAlmostEqual( A.width(), A[0].extent() )
        self.assertAlmostEqual( A.height(), A[1].extent() )
        
        self.assertEqual( A.dimensions(), ( A.width(), A.height() ) )
        #~ self.assertEqual( A.midpoint(), (A.min() + A.max())/2 )
        self.assertAlmostEqual(A.area(), A.width()*A.height())
        #TODO export EPSILON from 2geom
        if A.area() > 0:
            self.assertFalse(A.has_zero_area())
        else:
            self.assertTrue(A.has_zero_area())
        self.assertAlmostEqual(A.max_extent(), max(A.width(), A.height()))
        self.assertGreaterEqual(A.max_extent(), A.min_extent())
        
        bmin = B.min()
        bmax = B.max()

        pdiag = sqrt((amax[0]-amin[0])**2+(amax[1]-amin[1])**2)

        B.set_min(A.midpoint())
        B.set_max(A.midpoint())

        self.assertTrue(B.has_zero_area())
        
        B.expand_by(A.min_extent()/3.0)
        
        self.assertTrue(A.contains_rect(B))
        self.assertTrue(A.intersects(B))
        self.assertTrue(B.intersects(A))
        self.assertFalse(B.contains_rect(A))
        
        self.assertTrue(A.contains(A.midpoint()))
        self.assertFalse(A.contains( (A.midpoint()[0]*3, A.midpoint()[1]*3) ))
        
        A.union_with(B)
        
        self.assertEqual( A.min(), amin )
        
        B.set_left(bmin[0])
        B.set_top(bmin[1])
        B.set_right(bmax[0])
        B.set_bottom(bmax[1])
        
        self.assertEqual(B.min(), bmin)
        self.assertEqual(B.max(), bmax)
        
        B.expand_to( (0, 0) )
        self.assertEqual((0, 0), B.min())

        B.expand_by(*bmax)
        self.assertEqual(bmax, (- (B.min()[0]), - (B.min()[1])) )
        
        B.expand_by(-bmax[0], -bmax[1])
        self.assertEqual(B.max(), bmax)
        
        self.assertEqual( (A+B.min()).max()[0], A.max()[0] + B.min()[0] )
        
        #~ self.assertEqual( (A-B.max()).min(), A.min() - B.max() )
        
        self.assertEqual( A|A, A )
        
        self.assertFalse( A != A )

        B.set_left(bmin[0])
        B.set_top(bmin[1])
        B.set_right(bmax[0])
        B.set_bottom(bmax[1])
        
        #~ self.assertAlmostEqual(Rect.distance(Point(), A), A.min().length())
        #~ self.assertAlmostEqual(Rect.distanceSq(B.min(), A), Rect.distance(B.min(), A)**2 )


        
    def rect_basic(self, P, Q):
        pmin = P.min()
        pmax = P.max()
        #for simplicity
        self.assertTrue(pmin.x > 0)
        self.assertTrue(pmin.y > 0)
        
        self.assertAlmostEqual(pmin.x, P[0].min())
        self.assertAlmostEqual(pmax.y, P[1].max())
        self.assertEqual(pmin, P.corner(0))
        
        self.assertEqual(pmin, Point(P.left(), P.top()))
        self.assertEqual(pmax, Point(P.right(), P.bottom()))
        
        self.assertAlmostEqual( P.width(), P[0].extent() )
        self.assertAlmostEqual( P.height(), P[1].extent() )
        
        self.assertAlmostEqual( P.aspect_ratio(), P.width()/P.height() )
        self.assertEqual( P.dimensions(), Point( P.width(), P.height() ) )
        self.assertEqual( P.midpoint(), (P.min() + P.max())/2 )
        self.assertAlmostEqual(P.area(), P.width()*P.height())
        #TODO export EPSILON from 2geom
        if P.area() > 1e-7:
            self.assertFalse(P.has_zero_area())
            self.assertTrue(P.has_zero_area(P.area()))
        else:
            self.assertTrue(P.has_zero_area())
        self.assertAlmostEqual(P.max_extent(), max(P.width(), P.height()))
        self.assertGreaterEqual(P.max_extent(), P.min_extent())
        
        qmin = Q.min()
        qmax = Q.max()
        
        pdiag = (pmax-pmin).length()
        
        Q.set_min(P.midpoint())
        Q.set_max(P.midpoint())
        self.assertTrue(Q.has_zero_area())
        
        #print P,Q
        Q.expand_by(P.min_extent()/3.0)
         
        #print P, Q
        
        self.assertTrue(P.contains_rect(Q))
        self.assertTrue(P.intersects(Q))
        self.assertTrue(Q.intersects(P))
        self.assertFalse(Q.contains_rect(P))
        
        self.assertTrue(P.interior_contains_rect(Q))
        self.assertFalse(P.interior_contains_rect(P))
        self.assertTrue(P.interior_intersects(Q))
        self.assertTrue(P.interior_intersects(P))
        
        self.assertTrue(P.contains(P.midpoint()))
        self.assertFalse(P.contains(P.midpoint()*3))
        
        P.union_with(Q)
        
        self.assertEqual( P.min(), pmin )
        
        Q.set_left(qmin.x)
        Q.set_top(qmin.y)
        Q.set_right(qmax.x)
        Q.set_bottom(qmax.y)
        
        self.assertEqual(Q.min(), qmin)
        self.assertEqual(Q.max(), qmax)
        
        Q.expand_to( Point() )
        self.assertEqual(Point(), Q.min())
        Q.expand_by(qmax)
        self.assertEqual(qmax, -Q.min())
        
        Q.expand_by(-qmax.x, -qmax.y)
        self.assertEqual(Q.max(), qmax)
        
        self.assertEqual( (P+Q.min()).max(), P.max() + Q.min() )
        
        self.assertEqual( (P-Q.max()).min(), P.min() - Q.max() )
        
        self.assertEqual( P|P, P )
        
        self.assertFalse( P != P )

        Q.set_left(qmin.x)
        Q.set_top(qmin.y)
        Q.set_right(qmax.x)
        Q.set_bottom(qmax.y)
        
        self.assertAlmostEqual(Rect.distance(Point(), P), P.min().length())
        self.assertAlmostEqual(Rect.distanceSq(Q.min(), P), Rect.distance(Q.min(), P)**2 )
        
        self.assertEqual(P.round_outwards()[0], P[0].round_outwards())
        if P.round_inwards():
            self.assertEqual(P.round_inwards().Rect[1], P[1].round_inwards().Interval)
        
        
    def intrect_basic(self, P, Q):
        pmin = P.min()
        pmax = P.max()
        #for simplicity
        self.assertTrue(pmin.x > 0)
        self.assertTrue(pmin.y > 0)
        
        self.assertAlmostEqual(pmin.x, P[0].min())
        self.assertAlmostEqual(pmax.y, P[1].max())
        self.assertEqual(pmin, P.corner(0))
        
        self.assertEqual(pmin, IntPoint(P.left(), P.top()))
        self.assertEqual(pmax, IntPoint(P.right(), P.bottom()))
        
        self.assertAlmostEqual( P.width(), P[0].extent() )
        self.assertAlmostEqual( P.height(), P[1].extent() )
        
        self.assertAlmostEqual( P.aspect_ratio(), float(P.width())/float(P.height()) )
        self.assertEqual( P.dimensions(), IntPoint( P.width(), P.height() ) )
        self.assertEqual( P.midpoint().x, (P.min() + P.max()).x/2 )
        self.assertAlmostEqual(P.area(), P.width()*P.height())

        if P.area() > 0:
            self.assertFalse(P.has_zero_area())
        else:
            self.assertTrue(P.has_zero_area())
        self.assertAlmostEqual(P.max_extent(), max(P.width(), P.height()))
        self.assertGreaterEqual(P.max_extent(), P.min_extent())
        
        qmin = Q.min()
        qmax = Q.max()
        
        Q.set_min(P.midpoint())
        Q.set_max(P.midpoint())
        self.assertTrue(Q.has_zero_area())
        
        Q.expand_by(P.min_extent()/3.0)
        
        self.assertTrue(P.contains_rect(Q))
        self.assertTrue(P.intersects(Q))
        self.assertTrue(Q.intersects(P))
        self.assertFalse(Q.contains_rect(P))

        self.assertTrue(P.contains(P.midpoint()))
        self.assertFalse(P.contains(P.midpoint()+P.midpoint()+P.midpoint()))
        
        P.union_with(Q)
        
        self.assertEqual( P.min(), pmin )
        
        Q.set_left(qmin.x)
        Q.set_top(qmin.y)
        Q.set_right(qmax.x)
        Q.set_bottom(qmax.y)
        
        self.assertEqual(Q.min(), qmin)
        self.assertEqual(Q.max(), qmax)
        
        Q.expand_to( IntPoint() )
        self.assertEqual(IntPoint(), Q.min())
        Q.expand_by(qmax)
        self.assertEqual(qmax, IntPoint()-Q.min())
        
        Q.expand_by(-qmax.x, -qmax.y)
        self.assertEqual(Q.max(), qmax)
        
        self.assertEqual( (P+Q.min()).max(), P.max() + Q.min() )
        
        self.assertEqual( (P-Q.max()).min(), P.min() - Q.max() )
        
        self.assertEqual( P|P, P )
        
        self.assertFalse( P != P )

        Q.set_left(qmin.x)
        Q.set_top(qmin.y)
        Q.set_right(qmax.x)
        Q.set_bottom(qmax.y)



    def test_rect(self):
        
        P = Rect(0.298, 2, 4, 5)

        self.interval_basic(P[0], P[1])
        G = Rect(sqrt(2), sqrt(2), sqrt(3), sqrt(3))
        H = Rect.from_xywh(3.43232, 9.23214, 21.523, -0.31232)
        
        self.rect_basic(P, G)
        self.rect_basic(G, H)
        
        lst = [Point(randint(-100, 100), randint(-100, 100)) for i in range(10)]
   
        R = Rect.from_list(lst)
        
        for p in lst:
            self.assertTrue(R.contains(p))
            
        self.assertAlmostEqual(min(lst).y, R.min().y)
        self.assertAlmostEqual(max(lst).y, R.max().y)
        
        
        
    def test_optRect(self):
        
        P = OptRect(0.298, 2, 4, 5)
        self.interval_basic(P.Rect[0], P.Rect[1])
        
        
        G = Rect(sqrt(2), sqrt(2), sqrt(3), sqrt(3))
        H = OptRect.from_rect(G)

        self.rect_basic(P.Rect, G)
        
        lst = [Point(randint(-100, 100), randint(-100, 100)) for i in range(10)]

        R = OptRect.from_list(lst)

        for p in lst:
            self.assertTrue(R.Rect.contains(p))
            
        self.assertAlmostEqual(min(lst).y, R.Rect.min().y)
        self.assertAlmostEqual(max(lst).y, R.Rect.max().y)
        
        Q = OptRect()
        self.assertFalse(Q)
        self.assertTrue(P)

        self.assertTrue(Q.is_empty())
        self.assertFalse(P.is_empty())

        self.assertTrue(P.contains_rect( P ))
        self.assertTrue(P.contains_rect(Q))
        self.assertFalse(Q.contains_rect(P))
        self.assertFalse(P.intersects(Q))
        self.assertTrue(P.contains_rect(P.Rect))
        self.assertTrue(P.contains(P.Rect.midpoint())) 
        
        self.assertEqual(P, OptRect.from_rect(P))
        
        P.union_with(G)
        P.union_with(H)
        self.assertTrue(P.contains_rect(H))
        
        P.intersect_with(G)
        self.assertEqual(P, G)
        
        self.assertEqual( P|H, G )
        self.assertEqual( (P|R).Rect.min().x , min( P.Rect.min().x, R.Rect.min().x ))
        
        self.assertFalse(P & Q)
        self.assertEqual(P, P&P)
        
        self.assertEqual( P & (R | H), (P & R) | (P & H)  )
        
    def test_intRect(self):
        A = IntRect(2, 6, 9, 23)
        B = IntRect.from_intervals(IntInterval(1, 5), IntInterval(8, 9))
        C = IntRect.from_points(IntPoint(1, 8), IntPoint(5, 9))
        
        self.assertEqual(B, C)
        
        self.intrect_basic(A, B)
        self.intrect_basic(B, C)
        self.intrect_basic(C, A)
        
    def test_optIntRect(self):
        P = OptIntRect(1, 2, 4, 5)
        self.interval_basic(P.Rect[0], P.Rect[1])
        
        
        G = IntRect(2, 2, 3, 3)
        H = OptIntRect.from_rect(G)
        
        self.intrect_basic(P.Rect, G)
        
        lst = [IntPoint(randint(-100, 100), randint(-100, 100)) for i in range(10)]

        R = OptIntRect.from_list(lst)
        
        for p in lst:
            self.assertTrue(R.Rect.contains(p))
            
        self.assertAlmostEqual(min(lst).y, R.Rect.min().y)
        self.assertAlmostEqual(max(lst).y, R.Rect.max().y)
        
        Q = OptIntRect()
        self.assertFalse(Q)
        self.assertTrue(P)

        self.assertTrue(Q.is_empty())
        self.assertFalse(P.is_empty())

        self.assertTrue(P.contains_rect( P ))
        self.assertTrue(P.contains_rect( P.Rect ))
        self.assertTrue(P.contains_rect(Q))
        self.assertFalse(Q.contains_rect(P))
        self.assertFalse(P.intersects(Q))
        self.assertTrue(P.contains_rect(P.Rect))
        self.assertTrue(P.contains(P.Rect.midpoint())) 
        
        self.assertEqual(P, OptIntRect.from_rect(P))
        
        P.union_with(G)
        P.union_with(H)
        self.assertTrue(P.contains_rect(H))
        
        P.intersect_with(G)
        self.assertEqual(P, G)
        
        self.assertEqual( P|H, G )
        self.assertEqual( (P|R).Rect.min().x , min( P.Rect.min().x, R.Rect.min().x ))
        
        self.assertFalse(P & Q)
        self.assertEqual(P, P&P)
        
        self.assertEqual( P & (R | H), (P & R) | (P & H)  )

unittest.main()



from Tkinter import *

import math

import cy2geom
from cy2geom import Point, Path


def Nagon(N):
    """Return N-agon with side of length 1."""
    side = cy2geom.LineSegment(Point(-0.5, 0), Point(0.5, 0))
    angle = 2*math.pi/N
    distance_to_center = 0.5 / math.tan(math.pi/N)
    return Path.fromList(
        [ side.transformed( 
            cy2geom.Translate(Point(0, -distance_to_center))*
            cy2geom.Rotate(angle*i)) 
            for i in range(N)
        ], 
        stitching = Path.STITCH_DISCONTINUOUS, 
        closed = True )

def draw(c, dt=0.001, batch=10, scale=20, x_offset = 400, y_offset = 300):
    """Draw curve or path."""
    master = Tk()
    w = Canvas(master, width=800, height=600)
    w.pack()
    n = 0
    t = 0
    points = []
    
    if isinstance(c, Path):
        maxt = c.size_default()
    else:
        maxt = 1
    
    while (t < maxt):
        t = n*dt
        n+=1
        p = c(t)
        points.extend( [ p.x * scale + x_offset, p.y * scale + y_offset] )
        
    while points:
        draw_points = tuple(points[:batch*2])
        if len(points) == 2:
            break
        del points[:batch*2]
        
        l = w.create_line(*draw_points)
        w.grid()

    master.mainloop()



# * A simple toy to test the parser
# *
# * Copyright 2008 Aaron Spike <aaron@ekips.org>
# *
# * This library is free software; you can redistribute it and/or
# * modify it either under the terms of the GNU Lesser General Public
# * License version 2.1 as published by the Free Software Foundation
# * (the "LGPL") or, at your option, under the terms of the Mozilla
# * Public License Version 1.1 (the "MPL"). If you do not alter this
# * notice, a recipient may use your version of this file under either
# * the MPL or the LGPL.
# *
# * You should have received a copy of the LGPL along with this library
# * in the file COPYING-LGPL-2.1; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
# * You should have received a copy of the MPL along with this library
# * in the file COPYING-MPL-1.1
# *
# * The contents of this file are subject to the Mozilla Public License
# * Version 1.1 (the "License"); you may not use this file except in
# * compliance with the License. You may obtain a copy of the License at
# * http://www.mozilla.org/MPL/
# *
# * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY
# * OF ANY KIND, either express or implied. See the LGPL or the MPL for
# * the specific language governing rights and limitations.

import sys
sys.path.append(os.path.join(os.path.dirname(__file__), "..", "py2geom"))
import py2geom

class TestSink(py2geom.SVGPathSink):
    def __init__(self):
        py2geom.SVGPathSink.__init__(self)
        self.data = []
    def __str__(self):
        return ' '.join(self.data)
    def moveTo(self, p):
        x,y = p
        self.data.append('M %s, %s' % (x,y))
    def lineTo(self, p):
        x,y = p
        self.data.append('L %s, %s' % (x,y))
    def curveTo(self, c0, c1, p): 
        c0x,c0y = c0
        c1x,c1y = c1
        x,y = p
        self.data.append('C %s, %s %s, %s %s, %s' % (c0x,c0y,c1x,c1y,x,y))
    def quadTo(self, c, p): 
        cx,cy = c
        x,y = p
        self.data.append('Q %s, %s %s, %s' % (cx,cy,x,y))
    def arcTo(self, rx, ry, angle, large_arc, sweep, p): 
        x,y = p
        self.data.append('A %s, %s %s %i %i %s, %s' % (rx,ry,angle,large_arc,sweep,x,y))
    def closePath(self): 
        self.data.append('Z')
    def flush(self): 
        pass

def test_path(description, in_path, out_path):
    s = TestSink()
    py2geom.parse_svg_path(in_path, s)
    if str(s) == out_path:
        print 'Success: %s' % description
        return True
    else:
        print 'Error: %s' % description
        print '    given    "%s"' % in_path
        print '    got      "%s"' % str(s)
        print '    expected "%s"' % out_path
        return False

def run_tests(tests):
    successes = 0
    failures = 0 
    for description, in_path, out_path in tests:
        if test_path(description, in_path, out_path):
            successes += 1
        else:
            failures += 1
    print '=' * 20
    print 'Tests: %s' % (successes + failures)
    print 'Good:  %s' % successes
    print 'Bad:   %s' % failures

if __name__=='__main__':
    tests = [
        ('lineto', 'M 10,10 L 4,4', 'M 10.0, 10.0 L 4.0, 4.0'),
        ('implicit lineto', 'M 10,10 L 4,4 5,5 6,6', 'M 10.0, 10.0 L 4.0, 4.0 L 5.0, 5.0 L 6.0, 6.0'),
        ('implicit lineto after moveto', 'M1.2.3.4.5.6.7', 'M 1.2, 0.3 L 0.4, 0.5 L 0.6, 0.7'),
        ('arcto', 'M 300 150 A 150, 120, 30, 1, 0, 200 100', 'M 300.0, 150.0 A 150.0, 120.0 30.0 1 0 200.0, 100.0'),
    ]
    run_tests(tests)



from numbers import Number

from _common_decl cimport *
from cython.operator cimport dereference as deref

from _cy_affine cimport cy_Affine, get_Affine, is_transform


cdef class cy_GenericInterval:
    """
    Represents all numbers between min and max.

    Corresponds to GenericInterval in 2geom. min and max can be arbitrary
    python object, they just have to implement arithmetic operations and
    comparison - fractions.Fraction works. This is a bit experimental,
    it leak memory right now.
    """

    cdef GenericInterval[WrappedPyObject]* thisptr

    def __cinit__(self, u = 0, v = None):
        """Create GenericInterval from either one or two values."""
        if v is None:
            self.thisptr = new GenericInterval[WrappedPyObject]( WrappedPyObject(u) )
        else:
            self.thisptr = new GenericInterval[WrappedPyObject]( WrappedPyObject(u), WrappedPyObject(v) )

    def __str__(self):
        """str(self)"""
        return "[{}, {}]".format(self.min(), self.max())

    def __repr__(self):
        """repr(self)"""
        if self.is_singular():
            return "GenericInterval({})".format( str(self.min()) )
        return "GenericInterval({}, {})".format( str(self.min()) , str(self.max()) )

    def __dealloc__(self):
        del self.thisptr

    @classmethod
    def from_Interval(self, i):
        """Create GenericInterval with same minimum and maximum as argument."""
        return cy_GenericInterval( i.min(), i.max() )

    @classmethod
    def from_list(self, lst):
        """Create GenericInterval containing all values in list."""
        if len(lst) == 0:
            return cy_GenericInterval()
        ret = cy_GenericInterval(lst[0])
        for i in lst[1:]:
            ret.expand_to(i)
        return ret

    def min(self):
        """Return minimal value of interval."""
        return self.thisptr.min().getObj()

    def max(self):
        """Return maximal value of interval."""
        return self.thisptr.max().getObj()
    def extent(self):
        """Return difference between maximal and minimal value."""
        return self.thisptr.extent().getObj()

    def middle(self):
        """Return midpoint of interval."""
        return self.thisptr.middle().getObj()

    def is_singular(self):
        """Test for one-valued interval."""
        return self.thisptr.isSingular()

    def set_min(self, val):
        """Set minimal value."""
        self.thisptr.setMin( WrappedPyObject(val) )

    def set_max(self, val):
        """Set maximal value."""
        self.thisptr.setMax( WrappedPyObject(val) )

    def expand_to(self, val):
        """Create smallest superset of self containing value."""
        self.thisptr.expandTo( WrappedPyObject(val) )

    def expand_by(self, val):
        """Push both boundaries by value."""
        self.thisptr.expandBy( WrappedPyObject(val) )
    def union_with(self, cy_GenericInterval interval):
        """self = self | other"""
        self.thisptr.unionWith( deref(interval.thisptr) )

    def contains(self, other):
        """Check if interval contains value."""
        return self.thisptr.contains( WrappedPyObject(other) )

    def contains_interval(self, cy_GenericInterval other):
        """Check if interval contains every point of interval."""
        return self.thisptr.contains( deref(other.thisptr) )

    def intersects(self, cy_GenericInterval other):
        """Check for intersecting intervals."""
        return self.thisptr.intersects(deref( other.thisptr ))

    def __neg__(self):
        """Return interval with negated boundaries."""
        return wrap_GenericInterval(-deref(self.thisptr))

    def _add_pyobj(self, X):
        return wrap_GenericInterval(deref(self.thisptr) + WrappedPyObject(X) )
    def _sub_pyobj(self, X):
        return wrap_GenericInterval(deref(self.thisptr) - WrappedPyObject(X) )

    def _add_interval(self, cy_GenericInterval I):
        return wrap_GenericInterval(deref(self.thisptr)+deref(I.thisptr))
    def _sub_interval(self, cy_GenericInterval I):
        return wrap_GenericInterval(deref(self.thisptr)-deref(I.thisptr))

    def __add__(cy_GenericInterval self, other):
        """Add interval or value to self.

        Interval I+J consists of all values i+j such that i is in I and
        j is in J

        Interval I+x consists of all values i+x such that i is in I.
        """
        if isinstance(other, cy_GenericInterval):
            return self._add_interval(other)
        else:
            return self._add_pyobj(other)

    def __sub__(cy_GenericInterval self, other):
        """Substract interval or value.

        Interval I-J consists of all values i-j such that i is in I and
        j is in J

        Interval I-x consists of all values i-x such that i is in I.
        """
        if isinstance(other, cy_GenericInterval):
            return self._sub_interval(other)
        else:
            return self._sub_pyobj(other)

    def __or__(cy_GenericInterval self, cy_GenericInterval I):
        """Return a union of two intervals"""
        return wrap_GenericInterval(deref(self.thisptr)|deref(I.thisptr))

    def _eq(self, cy_GenericInterval other):
        return deref(self.thisptr)==deref(other.thisptr)

    def _neq(self, cy_GenericInterval other):
        return deref(self.thisptr)!=deref(other.thisptr)

    def __richcmp__(cy_GenericInterval self, other, op):
        """Intervals are not ordered."""
        if op == 2:
            return self._eq(other)
        elif op == 3:
            return self._neq(other)

cdef cy_GenericInterval wrap_GenericInterval(GenericInterval[WrappedPyObject] p):
    cdef GenericInterval[WrappedPyObject] * retp = new GenericInterval[WrappedPyObject](WrappedPyObject(0))
    retp[0] = p
    cdef cy_GenericInterval r = cy_GenericInterval.__new__(
                                        cy_GenericInterval, 0, 0)
    r.thisptr = retp
    return r


cdef class cy_GenericOptInterval:

    """Class representing optionally empty interval.

    Empty interval has False bool value, and using methods that require
    non-empty interval will result in ValueError. This is supposed to be
    used this way:

    >>> C = A & B
    >>> if C:
    >>>     print C.min()

    This class represents GenericOptInterval with python object boundaries.
    It tries to model behaviour of std::optional.
    """

    cdef GenericOptInterval[WrappedPyObject]* thisptr

    def __cinit__(self, u = None, v = None):
        """Create interval from boundaries.

        Using no arguments, you will end up with empty interval."""
        if u is None:
            self.thisptr = new GenericOptInterval[WrappedPyObject]()
        elif v is None:
            self.thisptr = new GenericOptInterval[WrappedPyObject](WrappedPyObject(u))
        else:
            self.thisptr = new GenericOptInterval[WrappedPyObject](WrappedPyObject(u), WrappedPyObject(v) )

    def __bool__(self):
        """Logical value of interval, False only for empty interval."""
        return not self.thisptr.isEmpty()

    def __str__(self):
        """str(self)"""
        if not self:
            return "[]"
        return "[{}, {}]".format(self.Interval.min(), self.Interval.max())

    def __repr__(self):
        """repr(self)"""
        if not self:
            return "GenericOptInterval()"
        if self.Interval.isSingular():
            return "GenericOptInterval({})".format( str(self.Interval.min()) )
        return "GenericOptInterval({}, {})".format( str(self.Interval.min()) , str(self.Interval.max()) )

    def __dealloc__(self):
        del self.thisptr

    @classmethod
    def from_Interval(self, i):
        """Create interval from existing interval."""
        if hasattr(i, "isEmpty"):
            if i.isEmpty():
                return cy_GenericOptInterval()
            else:
                return cy_GenericOptInterval.from_Interval(i.Interval)
        return cy_GenericOptInterval( i.min(), i.max() )

    @classmethod
    def from_list(self, lst):
        """Create interval containing all values in list.

        Empty list will result in empty interval."""
        if len(lst) == 0:
            return cy_GenericOptInterval()
        ret = cy_GenericOptInterval(lst[0])
        for i in lst[1:]:
            ret.Interval.expandTo(i)
        return ret

    property Interval:
        """Get underlying GenericInterval."""
        def __get__(self):
            if self.is_empty():
                raise ValueError("Interval is empty.")
            else:
                return wrap_GenericInterval(self.thisptr.get())

    def is_empty(self):
        """Check whether interval is empty set."""
        return self.thisptr.isEmpty()

    def union_with(self, cy_GenericOptInterval o):
        """self = self | other"""
        self.thisptr.unionWith( deref(o.thisptr) )

    def intersect_with(cy_GenericOptInterval self, cy_GenericOptInterval o):
        """self = self & other"""
        self.thisptr.intersectWith( deref(o.thisptr) )

    def __or__(cy_GenericOptInterval self, cy_GenericOptInterval o):
        """Return a union of two intervals."""
        return wrap_GenericOptInterval(deref(self.thisptr) | deref(o.thisptr))

    def __and__(cy_GenericOptInterval self, cy_GenericOptInterval o):
        """Return an intersection of two intervals."""
        return wrap_GenericOptInterval(deref(self.thisptr) & deref(o.thisptr))

    def __richcmp__(cy_GenericOptInterval self, cy_GenericOptInterval o, int op):
        """Intervals are not ordered."""
        if op == 2:
            return deref(self.thisptr) == deref(o.thisptr)
        elif op == 3:
            return deref(self.thisptr) != deref(o.thisptr)
        return NotImplemented


    def _get_Interval_method(self, name):
        def f(*args, **kwargs):
            if self.is_empty():
                raise ValueError("GenericOptInterval is empty.")
            else:
                return self.Interval.__getattribute__(name)(*args, **kwargs)
        return f

    def __getattr__(self, name):

        Interval_methods = set(['contains', 'contains_interval', 
        'expand_by', 'expand_to', 'extent', 'from_Interval', 'from_list', 
        'intersects', 'is_singular', 'max', 'middle', 'min', 'set_max', 
        'set_min', 'union_with'])

        if name in Interval_methods:
            return self._get_Interval_method(name)
        else:
            raise AttributeError("GenericOptInterval instance has no attribute \"{}\"".format(name))

    def _wrap_Interval_method(self, name, *args, **kwargs):
        if self.isEmpty():
            raise ValueError("GenericOptInterval is empty.")
        else:
            return self.Interval.__getattr__(name)(*args, **kwargs)

    #declaring these by hand, because they take fixed number of arguments,
    #which is enforced by cython

    def __neg__(self):
        """Return interval with negated boundaries."""
        return self._wrap_Interval_method("__sub__")

    def __add__(cy_Interval self, other):
        """Add interval or value to self.

        Interval I+J consists of all values i+j such that i is in I and
        j is in J

        Interval I+x consists of all values i+x such that i is in I.
        """
        return self._wrap_Interval_method("__add__", other)

    def __sub__(cy_Interval self, other):
        """Substract interval or value.

        Interval I-J consists of all values i-j such that i is in I and
        j is in J

        Interval I-x consists of all values i-x such that i is in I.
        """
        return self._wrap_Interval_method("__sub__", other)

cdef cy_GenericOptInterval wrap_GenericOptInterval(GenericOptInterval[WrappedPyObject] p):
    cdef GenericOptInterval[WrappedPyObject] * retp = new GenericOptInterval[WrappedPyObject]()
    retp[0] = p
    cdef cy_GenericOptInterval r = cy_GenericOptInterval.__new__(cy_GenericOptInterval)
    r.thisptr = retp
    return r


cdef class cy_Interval:

    """Class representing interval on real line.

    Corresponds to Interval class in 2geom.
    """

    def __cinit__(self, u = None, v = None):
        """Create interval from it's boundaries.

        One argument will create interval consisting that value, no
        arguments create Interval(0).
        """
        if u is None:
            self.thisptr = new Interval()
        elif v is None:
            self.thisptr = new Interval(<Coord>float(u))
        else:
            self.thisptr = new Interval(<Coord>float(u), <Coord>float(v))

    def __str__(self):
        """str(self)"""
        return "[{}, {}]".format(self.min(), self.max())

    def __repr__(self):
        """repr(self)"""
        if self.is_singular():
            return "Interval({})".format( str(self.min()) )
        return "Interval({}, {})".format( str(self.min()) , str(self.max()) )

    def __dealloc__(self):
        del self.thisptr

    @classmethod
    def from_Interval(c, i):
        """Create Interval with same boundaries as argument."""
        return cy_Interval( i.min(), i.max() )

    @classmethod
    def from_list(cls, lst):
        """Create interval containing all values in a list."""
        if len(lst) == 0:
            return cy_Interval()
        ret = cy_Interval(lst[0])
        for i in lst[1:]:
            ret.expand_to(i)
        return ret

    def min(self):
        """Return minimal boundary."""
        return self.thisptr.min()

    def max(self):
        """Return maximal boundary."""
        return self.thisptr.max()

    def extent(self):
        """Return length of interval."""
        return self.thisptr.extent()

    def middle(self):
        """Return middle value."""
        return self.thisptr.middle()

    def set_min(self, Coord val):
        """Set minimal value."""
        self.thisptr.setMin(val)

    def set_max(self, Coord val):
        """Set maximal value."""
        self.thisptr.setMax(val)

    def expand_to(self, Coord val):
        """Set self to smallest superset of set containing value."""
        self.thisptr.expandTo(val)

    def expand_by(self, Coord amount):
        """Move both boundaries by amount."""
        self.thisptr.expandBy(amount)

    def union_with(self, cy_Interval a):
        """self = self | other"""
        self.thisptr.unionWith(deref( a.thisptr ))

#    Not exposing this - deprecated
#    def __getitem__(self, unsigned int i):
#        return deref(self.thisptr)[i]

    def is_singular(self):
        """Test if interval contains only one value."""
        return self.thisptr.isSingular()

    def isFinite(self):
        """Test for finiteness of interval's extent."""
        return self.thisptr.isFinite()

    def contains(cy_Interval self, other):
        """Test if interval contains number."""
        return self.thisptr.contains(float(other))

    def contains_interval(cy_Interval self, cy_Interval other):
        """Test if interval contains another interval."""
        return self.thisptr.contains( deref(other.thisptr) )

    def intersects(self, cy_Interval val):
        """Test for intersection of intervals."""
        return self.thisptr.intersects(deref( val.thisptr ))

    def interior_contains(cy_Interval self, other):
        """Test if interior of iterval contains number."""
        return self.thisptr.interiorContains(float(other))

    def interior_contains_interval(cy_Interval self, cy_Interval other):
        """Test if interior of interval contains another interval."""
        return self.thisptr.interiorContains( <Interval &> deref(other.thisptr) )


    def interior_intersects(self, cy_Interval val):
        """Test for intersection of interiors of two points."""
        return self.thisptr.interiorIntersects(deref( val.thisptr ))

    def _cmp_Interval(cy_Interval self, cy_Interval other, op):
        if op == 2:
            return deref(self.thisptr) == deref(other.thisptr)
        elif op == 3:
            return deref(self.thisptr) != deref(other.thisptr)
    def _cmp_IntInterval(cy_Interval self, cy_IntInterval other, op):
        if op == 2:
            return deref(self.thisptr) == deref(other.thisptr)
        elif op == 3:
            return deref(self.thisptr) != deref(other.thisptr)

    def __richcmp__(cy_Interval self, other, op):
        """Intervals are not ordered."""
        if isinstance(other, cy_Interval):
            return self._cmp_Interval(other, op)
        elif isinstance(other, cy_IntInterval):
            return self._cmp_IntInterval(other, op)

    def __neg__(self):
        """Return interval with negated boundaries."""
        return wrap_Interval(-deref(self.thisptr))

    def _add_number(self, Coord X):
        return wrap_Interval(deref(self.thisptr)+X)
    def _sub_number(self, Coord X):
        return wrap_Interval(deref(self.thisptr)-X)
    def _mul_number(self, Coord X):
        return wrap_Interval(deref(self.thisptr)*X)

    def _add_interval(self, cy_Interval I):
        return wrap_Interval(deref(self.thisptr)+deref(I.thisptr))
    def _sub_interval(self, cy_Interval I):
        return wrap_Interval(deref(self.thisptr)-deref(I.thisptr))
    def _mul_interval(self, cy_Interval I):
        return wrap_Interval(deref(self.thisptr)*deref(I.thisptr))

    def __mul__(cy_Interval self, other):
        """Multiply interval by interval or number.

        Multiplying by number simply multiplies boundaries,
        multiplying intervals creates all values that can be written as
        product i*j of i in I and j in J.
        """
        if isinstance(other, Number):
            return self._mul_number(float(other))
        else:
            return self._mul_interval(other)

    def __add__(cy_Interval self, other):
        """Add interval or value to self.

        Interval I+J consists of all values i+j such that i is in I and
        j is in J

        Interval I+x consists of all values i+x such that i is in I.
        """
        if isinstance(other, Number):
            return self._add_number(float(other))
        else:
            return self._add_interval(other)

    def __sub__(cy_Interval self, other):
        """Substract interval or value.

        Interval I-J consists of all values i-j such that i is in I and
        j is in J

        Interval I-x consists of all values i-x such that i is in I.
        """
        if isinstance(other, Number):
            return self._sub_number(float(other))
        else:
            return self._sub_interval(other)

    def __div__(cy_Interval self, Coord s):
        """Divide boundaries by number."""
        return wrap_Interval(deref(self.thisptr)/s)

    def __or__(cy_Interval self, cy_Interval I):
        """Return union of two intervals."""
        return wrap_Interval(deref(self.thisptr)|deref(I.thisptr))

    def round_outwards(self):
        """Create the smallest IntIterval that is superset."""
        return wrap_IntInterval(self.thisptr.roundOutwards())

    def round_inwards(self):
        """Create the largest IntInterval that is subset."""
        return wrap_OptIntInterval(self.thisptr.roundInwards())

cdef cy_Interval wrap_Interval(Interval p):
    cdef Interval * retp = new Interval()
    retp[0] = p
    cdef cy_Interval r = cy_Interval.__new__(cy_Interval)
    r.thisptr = retp
    return r


cdef class cy_OptInterval:

    """Class representing optionally empty interval on real line.

    Empty interval has False bool value, and using methods that require
    non-empty interval will result in ValueError. This is supposed to be
    used this way:

    >>> C = A & B
    >>> if C:
    >>>     print C.min()

    This class represents OptInterval. It tries to model behaviour of
    std::optional.
    """

    def __cinit__(self, u = None, v = None):
        """Create optionally empty interval form it's endpoints.

        No arguments will result in empty interval.
        """
        if u is None:
            self.thisptr = new OptInterval()
        elif v is None:
            self.thisptr = new OptInterval(<Coord>float(u))
        else:
            self.thisptr = new OptInterval(<Coord>float(u), <Coord>float(v))

    def __bool__(self):
        """Only empty interval is False."""
        return not self.thisptr.isEmpty()

    def __str__(self):
        """str(self)"""
        if not self:
            return "[]"
        return "[{}, {}]".format(self.Interval.min(), self.Interval.max())

    def __repr__(self):
        """repr(self)"""
        if not self:
            return "OptInterval()"
        if self.Interval.isSingular():
            return "OptInterval({})".format( str(self.Interval.min()) )
        return "OptInterval({}, {})".format( str(self.Interval.min()) , str(self.Interval.max()) )

    def __dealloc__(self):
        del self.thisptr

    @classmethod
    def from_Interval(cls, i):
        """Create interval from other (possibly empty) interval."""
        if hasattr(i, "isEmpty"):
            if i.isEmpty():
                return cy_OptInterval()
            else:
                return cy_OptInterval.from_Interval(i.Interval)
        return cy_OptInterval( i.min(), i.max() )

    @classmethod
    def from_list(self, lst):
        """Create interval containing all values in list.

        Empty list will result in empty interval."""
        if len(lst) == 0:
            return cy_OptInterval()
        ret = cy_OptInterval(lst[0])
        for i in lst[1:]:
            ret.Interval.expandTo(i)
        return ret

    property Interval:
        """Get underlying Interval."""
        def __get__(self):
            if self.is_empty():
                raise ValueError("Interval is empty.")
            else:
                return wrap_Interval(self.thisptr.get())

    def is_empty(self):
        """Test for empty interval."""
        return self.thisptr.isEmpty()

    def union_with(self, cy_OptInterval o):
        """self = self | other"""
        self.thisptr.unionWith( deref(o.thisptr) )

    def intersect_with(cy_OptInterval self, cy_OptInterval o):
        """self = self & other"""
        self.thisptr.intersectWith( deref(o.thisptr) )

    def __or__(cy_OptInterval self, cy_OptInterval o):
        """Return union of intervals."""
        return wrap_OptInterval(deref(self.thisptr) | deref(o.thisptr))

    def __and__(cy_OptInterval self, cy_OptInterval o):
        """Return intersection of intervals."""
        return wrap_OptInterval(deref(self.thisptr) & deref(o.thisptr))

    def _get_Interval_method(self, name):
        def f(*args, **kwargs):
            if self.is_empty():
                raise ValueError("OptInterval is empty.")
            else:
                return self.Interval.__getattribute__(name)(*args, **kwargs)
        return f

    def __getattr__(self, name):

        Interval_methods = set(['contains', 'contains_interval', 'expand_by', 
        'expand_to', 'extent', 'from_Interval', 'from_list', 
        'interior_contains', 'interior_contains_interval', 
        'interior_intersects', 'intersects', 'isFinite', 'is_singular', 
        'max', 'middle', 'min', 'round_inwards', 'round_outwards', 
        'set_max', 'set_min', 'union_with'])

        if name in Interval_methods:
            return self._get_Interval_method(name)
        else:
            raise AttributeError("OptInterval instance has no attribute \"{}\"".format(name))

    def _wrap_Interval_method(self, name, *args, **kwargs):
        if self.isEmpty():
            raise ValueError("OptInterval is empty.")
        else:
            return self.Interval.__getattr__(name)(*args, **kwargs)

    #declaring these by hand, because they take fixed number of arguments,
    #which is enforced by cython

    def __neg__(self):
        """Return interval with negated boundaries."""
        return self._wrap_Interval_method("__sub__")

    def __mul__(cy_Interval self, other):
        """Multiply interval by interval or number.

        Multiplying by number simply multiplies boundaries,
        multiplying intervals creates all values that can be written as
        product i*j of i in I and j in J.
        """
        return self._wrap_Interval_method("__mul__", other)

    def __add__(cy_Interval self, other):
        """Add interval or value to self.

        Interval I+J consists of all values i+j such that i is in I and
        j is in J

        Interval I+x consists of all values i+x such that i is in I.
        """
        return self._wrap_Interval_method("__add__", other)

    def __sub__(cy_Interval self, other):
        """Substract interval or value.

        Interval I-J consists of all values i-j such that i is in I and
        j is in J

        Interval I-x consists of all values i-x such that i is in I.
        """
        return self._wrap_Interval_method("__sub__", other)

    def __div__(cy_Interval self, other):
        """Divide boundaries by number."""
        return self._wrap_Interval_method("__div__", other)

cdef cy_OptInterval wrap_OptInterval(OptInterval p):
    cdef OptInterval * retp = new OptInterval()
    retp[0] = p
    cdef cy_OptInterval r = cy_OptInterval.__new__(cy_OptInterval)
    r.thisptr = retp
    return r


cdef class cy_IntInterval:

    """Class representing interval of integers.

    Corresponds to IntInterval class in 2geom.
    """

    cdef IntInterval* thisptr

    def __cinit__(self, u = None, v = None):
        """Create interval from it's boundaries.

        One argument will create interval consisting that value, no
        arguments create IntInterval(0).
        """
        if u is None:
            self.thisptr = new IntInterval()
        elif v is None:
            self.thisptr = new IntInterval(<IntCoord>int(u))
        else:
            self.thisptr = new IntInterval(<IntCoord>int(u), <IntCoord>int(v))

    def __str__(self):
        """str(self)"""
        return "[{}, {}]".format(self.min(), self.max())

    def __repr__(self):
        """repr(self)"""
        if self.is_singular():
            return "IntInterval({})".format( str(self.min()) )
        return "IntInterval({}, {})".format( str(self.min()) , str(self.max()) )

    def __dealloc__(self):
        del self.thisptr

    @classmethod
    def from_Interval(cls, i):
        return cy_IntInterval( int(i.min()), int(i.max()) )

    @classmethod
    def from_list(cls, lst):
        if len(lst) == 0:
            return cy_IntInterval()
        ret = cy_IntInterval(lst[0])
        for i in lst[1:]:
            ret.expand_to(i)
        return ret

    def min(self):
        """Return minimal boundary."""
        return self.thisptr.min()

    def max(self):
        """Return maximal boundary."""
        return self.thisptr.max()

    def extent(self):
        """Return length of interval."""
        return self.thisptr.extent()

    def middle(self):
        """Return middle value."""
        return self.thisptr.middle()

    def set_min(self, IntCoord val):
        """Set minimal value."""
        self.thisptr.setMin(val)

    def set_max(self, IntCoord val):
        """Set maximal value."""
        self.thisptr.setMax(val)

    def expand_to(self, IntCoord val):
        """Set self to smallest superset of set containing value."""
        self.thisptr.expandTo(val)

    def expand_by(self, IntCoord amount):
        """Move both boundaries by amount."""
        self.thisptr.expandBy(amount)

    def union_with(self, cy_IntInterval a):
        """self = self | other"""
        self.thisptr.unionWith(deref( a.thisptr ))

#    Not exposing this - deprecated
#    def __getitem__(self, unsigned int i):
#        return deref(self.thisptr)[i]

    def is_singular(self):
        """Test if interval contains only one value."""
        return self.thisptr.isSingular()

    def contains(cy_IntInterval self, other):
        """Test if interval contains number."""
        return self.thisptr.contains(<IntCoord> int(other))

    def contains_interval(cy_IntInterval self, cy_IntInterval other):
        """Test if interval contains another interval."""
        return self.thisptr.contains( deref(other.thisptr) )

    def intersects(self, cy_IntInterval val):
        """Test for intersection with other interval."""
        return self.thisptr.intersects(deref( val.thisptr ))

    def __richcmp__(cy_IntInterval self, cy_IntInterval other, op):
        """Intervals are not ordered."""
        if op == 2:
            return deref(self.thisptr) == deref(other.thisptr)
        elif op == 3:
            return deref(self.thisptr) != deref(other.thisptr)

    def __neg__(self):
        """Negate interval's endpoints."""
        return wrap_IntInterval(-deref(self.thisptr))

    def _add_number(self, IntCoord X):
        return wrap_IntInterval(deref(self.thisptr)+X)
    def _sub_number(self, IntCoord X):
        return wrap_IntInterval(deref(self.thisptr)-X)

    def _add_interval(self, cy_IntInterval I):
        return wrap_IntInterval(deref(self.thisptr)+deref(I.thisptr))
    def _sub_interval(self, cy_IntInterval I):
        return wrap_IntInterval(deref(self.thisptr)-deref(I.thisptr))

    def __add__(cy_IntInterval self, other):
        """Add interval or value to self.

        Interval I+J consists of all values i+j such that i is in I and
        j is in J

        Interval I+x consists of all values i+x such that i is in I.
        """
        if isinstance(other, Number):
            return self._add_number(int(other))
        else:
            return self._add_interval(other)

    def __sub__(cy_IntInterval self, other):
        """Substract interval or value.

        Interval I-J consists of all values i-j such that i is in I and
        j is in J

        Interval I-x consists of all values i-x such that i is in I.
        """
        if isinstance(other, Number):
            return self._sub_number(int(other))
        else:
            return self._sub_interval(other)

    def __or__(cy_IntInterval self, cy_IntInterval I):
        """Return union of two intervals."""
        return wrap_IntInterval(deref(self.thisptr)|deref(I.thisptr))

cdef cy_IntInterval wrap_IntInterval(IntInterval p):
    cdef IntInterval * retp = new IntInterval()
    retp[0] = p
    cdef cy_IntInterval r = cy_IntInterval.__new__(cy_IntInterval)
    r.thisptr = retp
    return r

cdef class cy_OptIntInterval:

    """Class representing optionally empty interval of integers.

    Empty interval has False bool value, and using methods that require
    non-empty interval will result in ValueError. This is supposed to be
    used this way:

    >>> C = A & B
    >>> if C:
    >>>     print C.min()

    This class represents OptIntInterval. It tries to model behaviour of
    std::optional.
    """

    cdef OptIntInterval* thisptr

    def __cinit__(self, u = None, v = None):
        """Create optionally empty interval form it's endpoints.

        No arguments will result in empty interval.
        """
        if u is None:
            self.thisptr = new OptIntInterval()
        elif v is None:
            self.thisptr = new OptIntInterval(<IntCoord>int(u))
        else:
            self.thisptr = new OptIntInterval(<IntCoord>int(u), <IntCoord>int(v))

    def __bool__(self):
        """Only empty interval is False."""
        return not self.thisptr.isEmpty()

    def __str__(self):
        """str(self)"""
        if not self:
            return "[]"
        return "[{}, {}]".format(self.Interval.min(), self.Interval.max())

    def __repr__(self):
        """repr(self)"""
        if not self:
            return "OptIntInterval()"
        if self.Interval.isSingular():
            return "OptIntInterval({})".format( str(self.Interval.min()) )
        return "OptIntInterval({}, {})".format( str(self.Interval.min()) , str(self.Interval.max()) )

    def __dealloc__(self):
        del self.thisptr

    @classmethod
    def from_Interval(self, i):
        """Create interval from other (possibly empty) interval."""
        if hasattr(i, "isEmpty"):
            if i.isEmpty():
                return cy_OptIntInterval()
            else:
                return cy_OptIntInterval.from_Interval(i.Interval)
        return cy_OptIntInterval( i.min(), i.max() )

    @classmethod
    def from_list(self, lst):
        """Create interval containing all values in list.

        Empty list will result in empty interval."""
        if len(lst) == 0:
            return cy_OptIntInterval()
        ret = cy_OptIntInterval(lst[0])
        for i in lst[1:]:
            ret.Interval.expandTo(i)
        return ret

    property Interval:
        """Get underlying interval."""
        def __get__(self):
            return wrap_IntInterval(self.thisptr.get())

    def is_empty(self):
        """Test for empty interval."""
        return self.thisptr.isEmpty()

    def union_with(self, cy_OptIntInterval o):
        """self = self | other"""
        self.thisptr.unionWith( deref(o.thisptr) )

    def intersect_with(cy_OptIntInterval self, cy_OptIntInterval o):
        """self = self & other"""
        self.thisptr.intersectWith( deref(o.thisptr) )

    def __or__(cy_OptIntInterval self, cy_OptIntInterval o):
        """Return a union of two intervals."""
        return wrap_OptIntInterval(deref(self.thisptr) | deref(o.thisptr))

    def __and__(cy_OptIntInterval self, cy_OptIntInterval o):
        """Return an intersection of two intervals."""
        return wrap_OptIntInterval(deref(self.thisptr) & deref(o.thisptr))

    #TODO decide how to implement various combinations of comparisons!

    def _get_Interval_method(self, name):
        def f(*args, **kwargs):
            if self.is_empty():
                raise ValueError("OptInterval is empty.")
            else:
                return self.Interval.__getattribute__(name)(*args, **kwargs)
        return f

    def __getattr__(self, name):

        Interval_methods = set(['contains', 'contains_interval', 
        'expand_by', 'expand_to', 'extent', 'from_Interval', 'from_list', 
        'intersects', 'is_singular', 'max', 'middle', 'min', 'set_max', 
        'set_min', 'union_with'])

        if name in Interval_methods:
            return self._get_Interval_method(name)
        else:
            raise AttributeError("OptIntInterval instance has no attribute \"{}\"".format(name))

    def _wrap_Interval_method(self, name, *args, **kwargs):
        if self.isEmpty():
            raise ValueError("OptIntInterval is empty.")
        else:
            return self.Interval.__getattr__(name)(*args, **kwargs)

    #declaring these by hand, because they take fixed number of arguments,
    #which is enforced by cython

    def __neg__(self):
        """Negate interval's endpoints."""
        return self._wrap_Interval_method("__sub__")


    def __add__(cy_Interval self, other):
        """Add interval or value to self.

        Interval I+J consists of all values i+j such that i is in I and
        j is in J

        Interval I+x consists of all values i+x such that i is in I.
        """
        return self._wrap_Interval_method("__add__", other)

    def __sub__(cy_Interval self, other):
        """Substract interval or value.

        Interval I-J consists of all values i-j such that i is in I and
        j is in J

        Interval I-x consists of all values i-x such that i is in I.
        """
        return self._wrap_Interval_method("__sub__", other)

cdef cy_OptIntInterval wrap_OptIntInterval(OptIntInterval p):
    cdef OptIntInterval * retp = new OptIntInterval()
    retp[0] = p
    cdef cy_OptIntInterval r = cy_OptIntInterval.__new__(cy_OptIntInterval)
    r.thisptr = retp
    return r


cdef class cy_GenericRect:

    """Class representing axis aligned rectangle, with arbitrary corners.

    Plane in which the rectangle lies can have any object as a coordinates,
    as long as they implement arithmetic operations and comparison.

    This is a bit experimental, corresponds to GenericRect[C] templated
    with (wrapped) python object.
    """

    cdef GenericRect[WrappedPyObject]* thisptr

    def __cinit__(self, x0=0, y0=0, x1=0, y1=0):
        """Create rectangle from it's top-left and bottom-right corners."""
        self.thisptr = new GenericRect[WrappedPyObject](WrappedPyObject(x0),
                                                        WrappedPyObject(y0),
                                                        WrappedPyObject(x1),
                                                        WrappedPyObject(y1))

    def __str__(self):
        """str(self)"""
        return "Rectangle with dimensions {}, topleft point {}".format(
            str(self.dimensions()),
            str(self.min()))

    def __repr__(self):
        """repr(self)"""
        return "Rect({}, {}, {}, {})".format( str(self.left()),
                                              str(self.top()),
                                              str(self.right()),
                                              str(self.bottom()) )


    def __dealloc__(self):
        del self.thisptr

    @classmethod
    def from_intervals(self, I, J):
        """Create rectangle from two intervals.

        First interval corresponds to side parallel with x-axis,
        second one with side parallel with y-axis."""
        return cy_GenericRect(I.min(), I.max(), J.min(), J.max())

    @classmethod
    def from_list(cls, lst):
        """Create rectangle containing all points in list.

        These points are represented simply by 2-tuples.
        """
        ret = cy_GenericRect()
        for a in lst:
            ret.expand_to(a)
        return ret

    @classmethod
    def from_xywh(cls, x, y, w, h):
        """Create rectangle from topleft point and dimensions."""
        return wrap_GenericRect( from_xywh(WrappedPyObject(x),
                                        WrappedPyObject(y),
                                        WrappedPyObject(w),
                                        WrappedPyObject(h)))

    def __getitem__(self, Dim2 d):
        """self[i]"""
        return wrap_GenericInterval( deref(self.thisptr)[d] )

    def min(self):
        """Get top-left point."""
        return wrap_PyPoint( self.thisptr.min() )

    def max(self):
        """Get bottom-right point."""
        return wrap_PyPoint( self.thisptr.max() )

    def corner(self, unsigned int i):
        """Get corners (modulo) indexed from 0 to 3."""
        return wrap_PyPoint( self.thisptr.corner(i) )

    def top(self):
        """Get top coordinate."""
        return self.thisptr.top().getObj()

    def bottom(self):
        """Get bottom coordinate."""
        return self.thisptr.bottom().getObj()

    def left(self):
        """Get left coordinate."""
        return self.thisptr.left().getObj()

    def right(self):
        """Get right coordinate."""
        return self.thisptr.right().getObj()

    def width(self):
        """Get width."""
        return self.thisptr.width().getObj()

    def height(self):
        """Get height."""
        return self.thisptr.height().getObj()

    #For some reason, Cpp aspectRatio returns Coord.
    def aspectRatio(self):
        """Get ratio between width and height."""
        return float(self.width())/float(self.height())

    def dimensions(self):
        """Get dimensions as tuple."""
        return wrap_PyPoint( self.thisptr.dimensions() )

    def midpoint(self):
        """Get midpoint as tuple."""
        return wrap_PyPoint( self.thisptr.midpoint() )

    def area(self):
        """Get area."""
        return self.thisptr.area().getObj()

    def has_zero_area(self):
        """Test for area being zero."""
        return self.thisptr.hasZeroArea()

    def max_extent(self):
        """Get bigger value from width, height."""
        return self.thisptr.maxExtent().getObj()

    def min_extent(self):
        """Get smaller value from width, height."""
        return self.thisptr.minExtent().getObj()

    def intersects(self, cy_GenericRect r):
        """Check if rectangle intersects another rectangle."""
        return self.thisptr.intersects(deref( r.thisptr ))

    def contains(self, r):
        """Check if rectangle contains point represented as tuple."""
        if not isinstance(r, tuple):
            raise TypeError("Tuple required to create point.")
        return self.thisptr.contains( make_PyPoint(r) )

    def contains_rect(self, cy_GenericRect r):
        """Check if rectangle contains another rect."""
        return self.thisptr.contains( deref(r.thisptr) )

    def set_left(self, val):
        """Set left coordinate."""
        self.thisptr.setLeft( WrappedPyObject(val) )

    def set_right(self, val):
        """Set right coordinate."""
        self.thisptr.setRight( WrappedPyObject(val) )

    def set_top(self, val):
        """Set top coordinate."""
        self.thisptr.setTop( WrappedPyObject(val) )

    def set_bottom(self, val):
        """Set bottom coordinate."""
        self.thisptr.setBottom( WrappedPyObject(val) )

    def set_min(self, p):
        """Set top-left point."""
        self.thisptr.setMin(make_PyPoint(p))

    def set_max(self, p):
        """Set bottom-right point."""
        self.thisptr.setMax(make_PyPoint(p))

    def expand_to(self, p):
        """Expand rectangle to contain point represented as tuple."""
        self.thisptr.expandTo(make_PyPoint(p))

    def union_with(self, cy_GenericRect b):
        """self = self | other."""
        self.thisptr.unionWith(deref( b.thisptr ))

    def expand_by(self, x, y = None):
        """Expand both intervals.

        Either expand them both by one value, or each by different value.
        """
        if y is None:
            self.thisptr.expandBy(WrappedPyObject(x))
        else:
            self.thisptr.expandBy(WrappedPyObject(x),
                                  WrappedPyObject(y))


    def __add__(cy_GenericRect self, p):
        """Offset rectangle by point."""
        return wrap_GenericRect( deref(self.thisptr) + make_PyPoint(p) )

    def __sub__(cy_GenericRect self, p):
        """Offset rectangle by -point."""
        return wrap_GenericRect( deref(self.thisptr) - make_PyPoint(p) )

    def __or__(cy_GenericRect self, cy_GenericRect o):
        """Return union of two rects - it's actually bounding rect of union."""
        return wrap_GenericRect( deref(self.thisptr) | deref( o.thisptr ))

    def __richcmp__(cy_GenericRect self, cy_GenericRect o, int op):
        """Rectangles are not ordered."""
        if op == 2:
            return deref(self.thisptr) == deref(o.thisptr)
        if op == 3:
            return deref(self.thisptr) != deref(o.thisptr)

cdef PyPoint make_PyPoint(p):
    return PyPoint( WrappedPyObject(p[0]), WrappedPyObject(p[1]) )

#D2[WrappedPyObject] is converted to tuple
cdef wrap_PyPoint(PyPoint p):
    return (p[0].getObj(), p[1].getObj())

cdef cy_GenericRect wrap_GenericRect(GenericRect[WrappedPyObject] p):
    cdef WrappedPyObject zero = WrappedPyObject(0)
    cdef GenericRect[WrappedPyObject] * retp = new GenericRect[WrappedPyObject](zero, zero, zero, zero)
    retp[0] = p
    cdef cy_GenericRect r = cy_GenericRect.__new__(cy_GenericRect)
    r.thisptr = retp
    return r


cdef class cy_Rect:

    """Class representing axis-aligned rectangle in 2D real plane.

    Corresponds to Rect class in 2geom."""

    def __cinit__(self, Coord x0=0, Coord y0=0, Coord x1=0, Coord y1=0):
        """Create Rect from coordinates of its top-left and bottom-right corners."""
        self.thisptr = new Rect(x0, y0, x1, y1)

    def __str__(self):
        """str(self)"""
        return "Rectangle with dimensions {}, topleft point {}".format(str(self.dimensions()), str(self.min()))

    def __repr__(self):
        """repr(self)"""
        return "Rect({}, {}, {}, {})".format( str(self.left()),
                                              str(self.top()),
                                              str(self.right()),
                                              str(self.bottom()))

    def __dealloc__(self):
        del self.thisptr

    @classmethod
    def from_points(cls, cy_Point p0, cy_Point p1):
        """Create rectangle from it's top-left and bottom-right corners."""
        return wrap_Rect( Rect(deref(p0.thisptr), deref(p1.thisptr)) )

    @classmethod
    def from_intervals(cls, I, J):
        """Create rectangle from two intervals representing its sides."""
        return wrap_Rect( Rect( float(I.min()),
                                float(J.min()),
                                float(I.max()),
                                float(J.max()) ) )

    @classmethod
    def from_list(cls, lst):
        """Create rectangle containing all points in list."""
        if lst == []:
            return cy_Rect()
        if len(lst) == 1:
            return cy_Rect.from_points(lst[0], lst[0])
        ret = cy_Rect.from_points(lst[0], lst[1])
        for a in lst:
            ret.expand_to(a)
        return ret

    @classmethod
    def from_xywh(cls, x, y, w, h):
        """Create rectangle from it's topleft point and dimensions."""
        return wrap_Rect( from_xywh(<Coord> x,
                                    <Coord> y,
                                    <Coord> w,
                                    <Coord> h) )

    @classmethod
    def infinite(self):
        """Create infinite rectangle."""
        return wrap_Rect(infinite())

    def __getitem__(self, Dim2 d):
        """self[d]"""
        return wrap_Interval( deref(self.thisptr)[d] )

    def min(self):
        """Get top-left point."""
        return wrap_Point( self.thisptr.min() )

    def max(self):
        """Get bottom-right point."""
        return wrap_Point( self.thisptr.max() )

    def corner(self, unsigned int i):
        """Get corners (modulo) indexed from 0 to 3."""
        return wrap_Point( self.thisptr.corner(i) )

    def top(self):
        """Get top coordinate."""
        return self.thisptr.top()

    def bottom(self):
        """Get bottom coordinate."""
        return self.thisptr.bottom()

    def left(self):
        """Get left coordinate."""
        return self.thisptr.left()

    def right(self):
        """Get right coordinate."""
        return self.thisptr.right()

    def width(self):
        """Get width."""
        return self.thisptr.width()

    def height(self):
        """Get height."""
        return self.thisptr.height()

    def aspect_ratio(self):
        """Get ratio between width and height."""
        return self.thisptr.aspectRatio()

    def dimensions(self):
        """Get dimensions as point."""
        return wrap_Point( self.thisptr.dimensions() )

    def midpoint(self):
        """Get midpoint."""
        return wrap_Point( self.thisptr.midpoint() )

    def area(self):
        """Get area."""
        return self.thisptr.area()

    def has_zero_area(self, Coord eps = EPSILON):
        """Test for area being zero."""
        return self.thisptr.hasZeroArea(eps)

    def max_extent(self):
        """Get bigger value from width, height."""
        return self.thisptr.maxExtent()

    def min_extent(self):
        """Get smaller value from width, height."""
        return self.thisptr.minExtent()

    def intersects(self, cy_Rect r):
        """Check if rectangle intersects another rectangle."""
        return self.thisptr.intersects(deref( r.thisptr ))

    def contains(self, cy_Point r):
        """Check if rectangle contains point."""
        return self.thisptr.contains( deref(r.thisptr) )

    def contains_rect(self, cy_Rect r):
        """Check if rectangle contains another rect."""
        return self.thisptr.contains( deref(r.thisptr) )

    def interior_intersects(self, cy_Rect r):
        """Check if interior of self intersects another rectangle."""
        return self.thisptr.interiorIntersects(deref( r.thisptr ))

    def interior_contains(self, cy_Point other):
        """Check if interior of self contains point."""
        return self.thisptr.interiorContains( deref( (<cy_Point> other).thisptr ) )

    def interior_contains_rect(self, other):
        """Check if interior of self contains another rectangle."""
        if isinstance(other, cy_Rect):
            return self.thisptr.interiorContains( deref( (<cy_Rect> other).thisptr ) )
        elif isinstance(other, cy_OptRect):
            return self.thisptr.interiorContains( deref( (<cy_OptRect> other).thisptr ) )

    def set_left(self, Coord val):
        """Set left coordinate."""
        self.thisptr.setLeft(val)

    def set_right(self, Coord val):
        """Set right coordinate."""
        self.thisptr.setRight(val)

    def set_top(self, Coord val):
        """Set top coordinate."""
        self.thisptr.setTop(val)

    def set_bottom(self, Coord val):
        """Set bottom coordinate."""
        self.thisptr.setBottom(val)

    def set_min(self, cy_Point p):
        """Set top-left point."""
        self.thisptr.setMin( deref( p.thisptr ) )

    def set_max(self, cy_Point p):
        """Set bottom-right point."""
        self.thisptr.setMax( deref( p.thisptr ))

    def expand_to(self, cy_Point p):
        """Expand rectangle to contain point represented as tuple."""
        self.thisptr.expandTo( deref( p.thisptr ) )

    def union_with(self, cy_Rect b):
        """self = self | other."""
        self.thisptr.unionWith(deref( b.thisptr ))

    def expand_by(cy_Rect self, x, y = None):
        """Expand both intervals.

        Either expand them both by one value, or each by different value.
        """
        if y is None:
            if isinstance(x, cy_Point):
                self.thisptr.expandBy( deref( (<cy_Point> x).thisptr ) )
            else:
                self.thisptr.expandBy( <Coord> x)
        else:
            self.thisptr.expandBy( <Coord> x,
                                   <Coord> y)

    def __add__(cy_Rect self, cy_Point p):
        """Offset rectangle by point."""
        return wrap_Rect( deref(self.thisptr) + deref( p.thisptr ) )

    def __sub__(cy_Rect self, cy_Point p):
        """Offset rectangle by -point."""
        return wrap_Rect( deref(self.thisptr) - deref( p.thisptr ) )

    def __mul__(cy_Rect self, t):
        """Apply transform to rectangle."""
        cdef Affine at
        if is_transform(t):
            at = get_Affine(t)
            return wrap_Rect( deref(self.thisptr) * at )

    def __or__(cy_Rect self, cy_Rect o):
        """Return union of two rects - it's actually bounding rect of union."""
        return wrap_Rect( deref(self.thisptr) | deref( o.thisptr ))

    def __richcmp__(cy_Rect self, o, int op):
        """Rectangles are not ordered."""
        if op == 2:
            if isinstance(o, cy_Rect):
                return deref(self.thisptr) == deref( (<cy_Rect> o).thisptr)
            elif isinstance(o, cy_IntRect):
                return deref(self.thisptr) == deref( (<cy_IntRect> o).thisptr)
        if op == 3:
            if isinstance(o, cy_Rect):
                return deref(self.thisptr) != deref( (<cy_Rect> o).thisptr)
            elif isinstance(o, cy_IntRect):
                return deref(self.thisptr) != deref( (<cy_IntRect> o).thisptr)

    def round_inwards(self):
        """Create OptIntRect rounding inwards."""
        return wrap_OptIntRect(self.thisptr.roundInwards())

    def round_outwards(self):
        """Create IntRect rounding outwards."""
        return wrap_IntRect(self.thisptr.roundOutwards())

    @classmethod
    def distanceSq(cls, cy_Point p, cy_Rect rect):
        """Compute square of distance between point and rectangle."""
        return distanceSq( deref(p.thisptr), deref(rect.thisptr) )

    @classmethod
    def distance(cls, cy_Point p, cy_Rect rect):
        """Compute distance between point and rectangle."""
        return distance( deref(p.thisptr), deref(rect.thisptr) )

cdef cy_Rect wrap_Rect(Rect p):
    cdef Rect* retp = new Rect()
    retp[0] = p
    cdef cy_Rect r = cy_Rect.__new__(cy_Rect)
    r.thisptr = retp
    return r


cdef class cy_OptRect:

    """Class representing optionally empty rect in real plane.

    This class corresponds to OptRect in 2geom, and it tries to mimic
    the behaviour of std::optional. In addition to OptRect methods,
    this class passes calls for Rect methods to underlying Rect class,
    or throws ValueError when it's empty.
    """

    def __cinit__(self, x0=None, y0=None, x1=None, y1=None):
        """Create OptRect from coordinates of top-left and bottom-right corners.

        No arguments will result in empty rectangle.
        """
        if x0 is None:
            self.thisptr = new OptRect()
        else:
            self.thisptr = new OptRect( float(x0),
                                        float(y0),
                                        float(x1),
                                        float(y1) )

    def __str__(self):
        """str(self)"""
        if self.is_empty():
            return "Empty OptRect."
        return "OptRect with dimensions {}, topleft point {}".format(str(self.dimensions()), str(self.min()))

    def __repr__(self):
        """repr(self)"""
        if self.is_empty():
            return "OptRect()"
        return "OptRect({}, {}, {}, {})".format( str(self.left()),
                                                 str(self.top()),
                                                 str(self.right()),
                                                 str(self.bottom()))

    def __dealloc__(self):
        del self.thisptr

    @classmethod
    def from_points(cls, cy_Point p0, cy_Point p1):
        """Create rectangle from it's top-left and bottom-right corners."""
        return wrap_OptRect( OptRect(deref(p0.thisptr), deref(p1.thisptr)) )

    @classmethod
    def from_intervals(cls, I, J):
        """Create rectangle from two intervals representing its sides."""
        if hasattr(I, "isEmpty"):
            if I.isEmpty():
                return cy_OptRect()

        if hasattr(J, "isEmpty"):
            if J.isEmpty():
                return cy_OptRect()

        return wrap_OptRect( OptRect( float(I.min()),
                                      float(J.min()),
                                      float(I.max()),
                                      float(J.max()) ) )

    @classmethod
    def from_rect(cls, r):
        """Create OptRect from other, possibly empty, rectangle."""
        if hasattr(r, "isEmpty"):
            if r.isEmpty():
                return cy_OptRect()

        return cy_OptRect(  r.min().x,
                            r.min().y,
                            r.max().x,
                            r.max().y )

    @classmethod
    def from_list(cls, lst):
        """Create OptRect containing all points in the list.

        Empty list will result in empty OptRect.
        """
        if lst == []:
            return cy_OptRect()
        if len(lst) == 1:
            return cy_OptRect.from_points(lst[0], lst[0])
        ret = cy_OptRect.from_points(lst[0], lst[1])
        for a in lst:
            ret.expand_to(a)
        return ret

    property Rect:
        """Get underlying Rect."""
        def __get__(self):
            if self.is_empty():
                raise ValueError("Rect is empty.")
            else:
                return wrap_Rect(self.thisptr.get())

    def __bool__(self):
        """OptRect is False only when it's empty."""
        return not self.thisptr.isEmpty()

    def is_empty(self):
        """Check for OptRect containing no points."""
        return self.thisptr.isEmpty()

    def intersects(self, other):
        """Check if rectangle intersects another rectangle."""
        if isinstance(other, cy_Rect):
            return self.thisptr.intersects( deref( (<cy_Rect> other).thisptr ) )
        elif isinstance(other, cy_OptRect):
            return self.thisptr.intersects( deref( (<cy_OptRect> other).thisptr ) )

    def contains(self, cy_Point r):
        """Check if rectangle contains point."""
        return self.thisptr.contains( deref(r.thisptr) )

    def contains_rect(self, other):
        """Check if rectangle contains another rect."""
        if isinstance(other, cy_Rect):
            return self.thisptr.contains( deref( (<cy_Rect> other).thisptr ) )
        elif isinstance(other, cy_OptRect):
            return self.thisptr.contains( deref( (<cy_OptRect> other).thisptr ) )

    def union_with(self, other):
        """self = self | other."""
        if isinstance(other, cy_Rect):
            self.thisptr.unionWith( deref( (<cy_Rect> other).thisptr ) )
        elif isinstance(other, cy_OptRect):
            self.thisptr.unionWith( deref( (<cy_OptRect> other).thisptr ) )

    def intersect_with(self, other):
        """self = self & other."""
        if isinstance(other, cy_Rect):
            self.thisptr.intersectWith( deref( (<cy_Rect> other).thisptr ) )
        elif isinstance(other, cy_OptRect):
            self.thisptr.intersectWith( deref( (<cy_OptRect> other).thisptr ) )

    def expand_to(self, cy_Point p):
        """Expand rectangle to contain point represented as tuple."""
        self.thisptr.expandTo( deref(p.thisptr) )

    def __or__(cy_OptRect self, cy_OptRect other):
        """Return union of two rects - it's actually bounding rect of union."""
        return wrap_OptRect( deref(self.thisptr) | deref(other.thisptr) )

    def __and__(cy_OptRect self, other):
        """Return intersection of two rectangles."""
        if isinstance(other, cy_Rect):
            return wrap_OptRect( deref(self.thisptr) & deref( (<cy_Rect> other).thisptr) )
        elif isinstance(other, cy_OptRect):
            return wrap_OptRect( deref(self.thisptr) & deref( (<cy_OptRect> other).thisptr) )

    def __richcmp__(cy_OptRect self, other, op):
        """Rectangles are not ordered."""
        if op == 2:
            if isinstance(other, cy_Rect):
                return deref(self.thisptr) == deref( (<cy_Rect> other).thisptr )
            elif isinstance(other, cy_OptRect):
                return deref(self.thisptr) == deref( (<cy_OptRect> other).thisptr )
        elif op == 3:
            if isinstance(other, cy_Rect):
                return deref(self.thisptr) != deref( (<cy_Rect> other).thisptr )
            elif isinstance(other, cy_OptRect):
                return deref(self.thisptr) != deref( (<cy_OptRect> other).thisptr )
        return NotImplemented

    def _get_Rect_method(self, name):
        def f(*args, **kwargs):
            if self.is_empty():
                raise ValueError("OptRect is empty.")
            else:
                return self.Rect.__getattribute__(name)(*args, **kwargs)
        return f

    def __getattr__(self, name):
        Rect_methods = set(['area', 'aspectRatio', 'bottom', 'contains',
        'contains_rect', 'corner', 'dimensions', 'distance', 'distanceSq',
        'expand_by', 'expand_to', 'has_zero_area', 'height', 'infinite', 
        'interior_contains', 'interior_contains_rect',
        'interior_intersects', 'intersects', 'left', 'max', 'max_extent',
        'midpoint', 'min', 'min_extent', 'right', 'round_inwards',
        'round_outwards', 'set_bottom', 'set_left', 'set_max', 'set_min',
        'set_right', 'set_top', 'top', 'union_with', 'width'])

        if name in Rect_methods:
            return self._get_Rect_method(name)
        else:
            raise AttributeError("OptRect instance has no attribute \"{}\"".format(name))

    def _wrap_Rect_method(self, name, *args, **kwargs):
        if self.isEmpty():
            raise ValueError("OptRect is empty.")
        else:
            return self.Rect.__getattr__(name)(*args, **kwargs)

    #declaring these by hand, because they take fixed number of arguments,
    #which is enforced by cython

    def __getitem__(self, i):
        """self[d]"""
        return self._wrap_Rect_method("__getitem__", i)

    def __add__(self, other):
        """Offset rectangle by point."""
        return self._wrap_Rect_method("__add__", other)

    def __mul__(self, other):
        """Apply transform to rectangle."""
        return self._wrap_Rect_method("__mul__", other)

    def __sub__(self, other):
        """Offset rectangle by -point."""
        return self._wrap_Rect_method("__sub__", other)


cdef cy_OptRect wrap_OptRect(OptRect p):
    cdef OptRect* retp = new OptRect()
    retp[0] = p
    cdef cy_OptRect r = cy_OptRect.__new__(cy_OptRect)
    r.thisptr = retp
    return r



cdef class cy_IntRect:

    """Class representing axis-aligned rectangle in 2D with integer coordinates.

    Corresponds to IntRect class (typedef) in 2geom."""

    cdef IntRect* thisptr

    def __cinit__(self, IntCoord x0=0, IntCoord y0=0, IntCoord x1=0, IntCoord y1=0):
        """Create IntRect from coordinates of its top-left and bottom-right corners."""
        self.thisptr = new IntRect(x0, y0, x1, y1)

    def __str__(self):
        """str(self)"""
        return "IntRect with dimensions {}, topleft point {}".format(
            str(self.dimensions()),
            str(self.min()))

    def __repr__(self):
        """repr(self)"""
        return "IntRect({}, {}, {}, {})".format( str(self.left()),
                                                 str(self.top()),
                                                 str(self.right()),
                                                 str(self.bottom()))

    def __dealloc__(self):
        del self.thisptr

    @classmethod
    def from_points(cls, cy_IntPoint p0, cy_IntPoint p1):
        """Create rectangle from it's top-left and bottom-right corners."""
        return wrap_IntRect( IntRect(deref(p0.thisptr), deref(p1.thisptr)) )

    @classmethod
    def from_intervals(cls, I, J):
        """Create rectangle from two intervals representing its sides."""
        return cy_IntRect(  int(I.min()),
                            int(J.min()),
                            int(I.max()),
                            int(J.max()) )

    @classmethod
    def from_list(cls, lst):
        """Create rectangle containing all points in list."""
        if lst == []:
            return cy_IntRect()
        if len(lst) == 1:
            return cy_IntRect(lst[0], lst[0])
        ret = cy_IntRect(lst[0], lst[1])
        for a in lst:
            ret.expand_to(a)
        return ret

    #didn't manage to declare from_xywh for IntRect
    @classmethod
    def from_xywh(cls, x, y, w, h):
        """Create rectangle from it's topleft point and dimensions."""
        return cy_IntRect(int(x),
                          int(y),
                          int(x) + int(w),
                          int(y) + int(h) )

    def __getitem__(self, Dim2 d):
        """self[d]"""
        return wrap_IntInterval( deref(self.thisptr)[d] )

    def min(self):
        """Get top-left point."""
        return wrap_IntPoint( self.thisptr.i_min() )

    def max(self):
        """Get bottom-right point."""
        return wrap_IntPoint( self.thisptr.i_max() )

    def corner(self, unsigned int i):
        """Get corners (modulo) indexed from 0 to 3."""
        return wrap_IntPoint( self.thisptr.i_corner(i) )

    def top(self):
        """Get top coordinate."""
        return self.thisptr.top()

    def bottom(self):
        """Get bottom coordinate."""
        return self.thisptr.bottom()

    def left(self):
        """Get left coordinate."""
        return self.thisptr.left()

    def right(self):
        """Get right coordinate."""
        return self.thisptr.right()

    def width(self):
        """Get width."""
        return self.thisptr.width()

    def height(self):
        """Get height."""
        return self.thisptr.height()

    def aspect_ratio(self):
        """Get ratio between width and height."""
        return self.thisptr.aspectRatio()

    def dimensions(self):
        """Get dimensions as IntPoint."""
        return wrap_IntPoint( self.thisptr.i_dimensions() )

    def midpoint(self):
        """Get midpoint."""
        return wrap_IntPoint( self.thisptr.i_midpoint() )

    def area(self):
        """Get area."""
        return self.thisptr.area()

    def has_zero_area(self):
        """Test for area being zero."""
        return self.thisptr.hasZeroArea()

    def max_extent(self):
        """Get bigger value from width, height."""
        return self.thisptr.maxExtent()

    def min_extent(self):
        """Get smaller value from width, height."""
        return self.thisptr.minExtent()

    def intersects(self, cy_IntRect r):
        """Check if rectangle intersects another rectangle."""
        return self.thisptr.intersects(deref( r.thisptr ))

    def contains(self, cy_IntPoint r):
        """Check if rectangle contains point."""
        return self.thisptr.contains( deref(r.thisptr) )

    def contains_rect(self, cy_IntRect r):
        """Check if rectangle contains another rect."""
        return self.thisptr.contains( deref(r.thisptr) )

    def set_left(self, IntCoord val):
        """Set left coordinate."""
        self.thisptr.setLeft(val)

    def set_right(self, IntCoord val):
        """Set right coordinate."""
        self.thisptr.setRight(val)

    def set_top(self, IntCoord val):
        """Set top coordinate."""
        self.thisptr.setTop(val)

    def set_bottom(self, IntCoord val):
        """Set bottom coordinate."""
        self.thisptr.setBottom(val)

    def set_min(self, cy_IntPoint p):
        """Set top-left point."""
        self.thisptr.setMin( deref( p.thisptr ) )

    def set_max(self, cy_IntPoint p):
        """Set bottom-right point."""
        self.thisptr.setMax( deref( p.thisptr ))

    def expand_to(self, cy_IntPoint p):
        """Expand rectangle to contain point represented as tuple."""
        self.thisptr.expandTo( deref( p.thisptr ) )

    def union_with(self, cy_IntRect b):
        """self = self | other."""
        self.thisptr.unionWith(deref( b.thisptr ))

    def expand_by(cy_IntRect self, x, y = None):
        """Expand both intervals.

        Either expand them both by one value, or each by different value.
        """
        if y is None:
            if isinstance(x, cy_IntPoint):
                self.thisptr.expandBy( deref( (<cy_IntPoint> x).thisptr ) )
            else:
                self.thisptr.expandBy( <IntCoord> x)
        else:
            self.thisptr.expandBy( <IntCoord> x,
                                   <IntCoord> y)

    def __add__(cy_IntRect self, cy_IntPoint p):
        """Offset rectangle by point."""
        return wrap_IntRect( deref(self.thisptr) + deref( p.thisptr ) )

    def __sub__(cy_IntRect self, cy_IntPoint p):
        """Offset rectangle by -point."""
        return wrap_IntRect( deref(self.thisptr) - deref( p.thisptr ) )

    def __or__(cy_IntRect self, cy_IntRect o):
        """Return union of two rects - it's actually bounding rect of union."""
        return wrap_IntRect( deref(self.thisptr) | deref( o.thisptr ))

    def __richcmp__(cy_IntRect self, cy_IntRect o, int op):
        """Rectangles are not ordered."""
        if op == 2:
            return deref(self.thisptr) == deref(o.thisptr)
        if op == 3:
            return deref(self.thisptr) != deref(o.thisptr)

cdef cy_IntRect wrap_IntRect(IntRect p):
    cdef IntRect* retp = new IntRect()
    retp[0] = p
    cdef cy_IntRect r = cy_IntRect.__new__(cy_IntRect)
    r.thisptr = retp
    return r



cdef class cy_OptIntRect:

    """Class representing optionally empty rect in with integer coordinates.

    This class corresponds to OptIntRect in 2geom, and it tries to mimic
    the behaviour of std::optional. In addition to OptIntRect methods,
    this class passes calls for IntRect methods to underlying IntRect class,
    or throws ValueError when it's empty.
    """

    cdef OptIntRect* thisptr
    
    def __cinit__(self, x0=None, y0=None, x1=None, y1=None):
        """Create OptIntRect from coordinates of top-left and bottom-right corners.

        No arguments will result in empty rectangle.
        """
        if x0 is None:
            self.thisptr = new OptIntRect()
        else:
            self.thisptr = new OptIntRect( int(x0),
                                        int(y0),
                                        int(x1),
                                        int(y1) )
    
    def __str__(self):
        """str(self)"""
        if self.isEmpty():
            return "Empty OptIntRect"
        return "OptIntRect with dimensions {}, topleft point {}".format(
            str(self.Rect.dimensions()), 
            str(self.Rect.min()))
    
    def __repr__(self):
        """repr(self)"""
        if self.isEmpty():
            return "OptIntRect()"
        return "OptIntRect({}, {}, {}, {})".format( str(self.Rect.left()),
                                                    str(self.Rect.top()),
                                                    str(self.Rect.right()),
                                                    str(self.Rect.bottom()))
    
    def __dealloc__(self):
        del self.thisptr
    
    @classmethod
    def from_points(cls, cy_IntPoint p0, cy_IntPoint p1):
        """Create rectangle from it's top-left and bottom-right corners."""
        return wrap_OptIntRect( OptIntRect(deref(p0.thisptr), deref(p1.thisptr)) )

    @classmethod
    def from_intervals(cls, I, J):
        """Create rectangle from two intervals representing its sides."""
        if hasattr(I, "isEmpty"):
            if I.isEmpty():
                return cy_OptIntRect()

        if hasattr(J, "isEmpty"):
            if J.isEmpty():
                return cy_OptIntRect()

        return wrap_OptIntRect( OptIntRect( int(I.min()),
                                            int(J.min()),
                                            int(I.max()),
                                            int(J.max()) ) )

    @classmethod
    def from_rect(cls, r):
        """Create OptIntRect from other, possibly empty, rectangle."""
        if hasattr(r, "isEmpty"):
            if r.isEmpty():
                return cy_OptIntRect()
        return cy_OptIntRect(   r.min().x,
                                r.min().y,
                                r.max().x,
                                r.max().y )

    @classmethod
    def from_list(cls, lst):
        """Create OptIntRect containing all points in the list.

        Empty list will result in empty OptIntRect.
        """
        if lst == []:
            return cy_OptIntRect()
        if len(lst) == 1:
            return cy_OptIntRect.from_points(lst[0], lst[0])
        ret = cy_OptIntRect.from_points(lst[0], lst[1])
        for a in lst:
            ret.expand_to(a)
        return ret

    property Rect:
        """Get underlying IntRect."""
        def __get__(self):
            return wrap_IntRect(self.thisptr.get())

    def __bool__(self):
        """OptIntRect is False only when it's empty."""
        return not self.thisptr.isEmpty()

    def is_empty(self):
        """Check for OptIntRect containing no points."""
        return self.thisptr.isEmpty()

    def intersects(cy_OptIntRect self, other):
        """Check if rectangle intersects another rectangle."""
        if isinstance(other, cy_IntRect):
            return self.thisptr.intersects( deref( (<cy_IntRect> other).thisptr ) )
        elif isinstance(other, cy_OptIntRect):
            return self.thisptr.intersects( deref( (<cy_OptIntRect> other).thisptr ) )
            
    def contains(self, cy_IntPoint other):
        """Check if rectangle contains point."""
        return self.thisptr.contains( deref(other.thisptr) )

    def contains_rect(cy_OptIntRect self, other):
        """Check if rectangle contains another rectangle."""
        if isinstance(other, cy_IntRect):
            return self.thisptr.contains( deref( (<cy_IntRect> other).thisptr ) )
        elif isinstance(other, cy_OptIntRect):
            return self.thisptr.contains( deref( (<cy_OptIntRect> other).thisptr ) )

            
    def union_with(cy_OptIntRect self, other):
        """self = self | other."""
        if isinstance(other, cy_IntRect):
            self.thisptr.unionWith( deref( (<cy_IntRect> other).thisptr ) )
        elif isinstance(other, cy_OptIntRect):
            self.thisptr.unionWith( deref( (<cy_OptIntRect> other).thisptr ) )
            
    def intersect_with(cy_OptIntRect self, other):
        """self = self & other."""
        if isinstance(other, cy_IntRect):
            self.thisptr.intersectWith( deref( (<cy_IntRect> other).thisptr ) )
        elif isinstance(other, cy_OptIntRect):
            self.thisptr.intersectWith( deref( (<cy_OptIntRect> other).thisptr ) )
            
    def expand_to(self, cy_IntPoint p):
        """Expand rectangle to contain point."""
        self.thisptr.expandTo( deref(p.thisptr) )

    def __or__(cy_OptIntRect self, cy_OptIntRect other):
        """Return union of two rects - it's actually bounding rect of union."""
        return wrap_OptIntRect( deref(self.thisptr) | deref(other.thisptr) )
        
    def __and__(cy_OptIntRect self, other):
        """Return intersection of two rectangles."""
        if isinstance(other, cy_IntRect):
            return wrap_OptIntRect( deref(self.thisptr) & deref( (<cy_IntRect> other).thisptr) )
        elif isinstance(other, cy_OptIntRect):
            return wrap_OptIntRect( deref(self.thisptr) & deref( (<cy_OptIntRect> other).thisptr) )

    def __richcmp__(cy_OptIntRect self, other, op):
        """Rectangles are not ordered."""
        if op == 2:
            if isinstance(other, cy_IntRect):
                return deref(self.thisptr) == deref( (<cy_IntRect> other).thisptr )
            elif isinstance(other, cy_OptIntRect):
                return deref(self.thisptr) == deref( (<cy_OptIntRect> other).thisptr )
        elif op == 3:
            if isinstance(other, cy_IntRect):
                return deref(self.thisptr) != deref( (<cy_IntRect> other).thisptr )
            elif isinstance(other, cy_OptIntRect):
                return deref(self.thisptr) != deref( (<cy_OptIntRect> other).thisptr )

    def _get_Rect_method(self, name):
        def f(*args, **kwargs):
            if self.is_empty():
                raise ValueError("OptIntRect is empty.")
            else:
                return self.Rect.__getattribute__(name)(*args, **kwargs)
        return f

    def __getattr__(self, name):

        Rect_methods = set(['area', 'aspect_ratio', 'bottom', 'contains', 
        'contains_rect', 'corner', 'dimensions', 'expand_by', 'expand_to', 
        'from_intervals', 'from_list', 'from_points', 'from_xywh', 
        'has_zero_area', 'height', 'intersects', 'left', 'max', 
        'max_extent', 'midpoint', 'min', 'min_extent', 'right', 
        'set_bottom', 'set_left', 'set_max', 'set_min', 'set_right', 
        'set_top', 'top', 'union_with', 'width'])

        if name in Rect_methods:
            return self._get_Rect_method(name)
        else:
            raise AttributeError("OptIntRect instance has no attribute \"{}\"".format(name))

    def _wrap_Rect_method(self, name, *args, **kwargs):
        if self.isEmpty():
            raise ValueError("OptIntRect is empty.")
        else:
            return self.Rect.__getattr__(name)(*args, **kwargs)

    #declaring these by hand, because they take fixed number of arguments,
    #which is enforced by cython

    def __getitem__(self, i):
        """self[d]"""
        return self._wrap_Rect_method("__getitem__", i)

    def __add__(self, other):
        """Offset rectangle by point."""
        return self._wrap_Rect_method("__add__", other)

    def __sub__(self, other):
        """Offset rectangle by -point."""
        return self._wrap_Rect_method("__sub__", other)

cdef cy_OptIntRect wrap_OptIntRect(OptIntRect p):
    cdef OptIntRect* retp = new OptIntRect()
    retp[0] = p
    cdef cy_OptIntRect r = cy_OptIntRect.__new__(cy_OptIntRect)
    r.thisptr = retp
    return r



#!/usr/bin/python

import py2geom as g

a = g.Point(1,2)
b = g.Point(31,2)
print a, b

point_fns_1 = ["L1", "L2", "L2sq", "LInfty", "is_zero", "is_unit_vector",
             "atan2", "rot90", 
             "unit_vector", "abs"]
point_fns_2 = ["dot", "angle_between", "distance", "distanceSq", "cross"]

for i in point_fns_1:
    print "%s:" % i, g.__dict__[i](a)
for i in point_fns_2:
    print "%s:" % i, g.__dict__[i](a,b)
print "a == b", a == b
print "Lerp:", g.lerp(0.3, a,b)

bo = g.BezOrd(2,3)
print bo
print bo.point_at(0.3)

print bo.reverse()

sn = g.sin(g.BezOrd(0.0,8.0),5)
print sn
print g.inverse(sn,10)
print list(sn)

r_sn = g.roots(sn)
print len(r_sn)
print list(r_sn)

bo = g.BezOrd(-1,1)
sb = g.SBasis()
print sb
print list(g.roots(sb))
sb.append(bo)
print list(g.roots(sb))
